<?php
/**
 * Class for the By Plugin page.

 * @package Freesoul Deactivate Plugins
 */

defined( 'ABSPATH' ) || exit; // Exit if accessed directly.

class FDP_By_Plugin_Page extends Eos_Fdp_Matrix_Page {

	public function before_section( $page_slug ) {
		?>
		<h2><?php echo esc_html( $this->title ); ?></h2>
		<p><?php echo esc_html( apply_filters( 'fdp_by_plugin_description', __( 'Disable specific plugins when other plugins are disabled.','eos-dp-pro' ) ) ); ?></p>
		<?php
		add_filter( 'fdp_allowed_backend_scripts', function( $scripts ) {
			$scripts[] = 'fdp-by-plugin';
			return $scripts;
		} );
		wp_enqueue_script( 'fdp-by-plugin', EOS_DP_PRO_PLUGIN_URL . '/admin/js/eos-dp-pro-by-plugin-1.0.0.js', array(), true );
		wp_localize_script(
			'fdp-by-plugin',
			'fdp_by_plugin_js',
			array(
				'ajaxurl'  => admin_url( 'admin-ajax.php' ),
				'page_slug' => esc_attr( $page_slug ),
				'action'   => 'eos_dp_pro_save_by_plugin_settings',
				'opts_key' => sanitize_key( $page_slug )
			)
		);
	}

	public function tableBody( $page_slug ) {
		$row = 0;
		$opts = eos_dp_get_option( sanitize_text_field( $this->page_slug ) );
		foreach ( $this->active_plugins as $plugin ) {
				$active  = $opt_array = false;
				$opt = isset( $opts[$plugin] ) ? $opts[$plugin] : false;
				if( $opt ) {
					$opt_array = array_filter( explode( ',', $opt ) );
				}
				if ( in_array( $plugin, array_keys( $this->plugins_by_dirs ) ) ) {
					$plugin_title = function_exists( 'eos_dp_get_plugin_name_by_slug' ) ? eos_dp_get_plugin_name_by_slug( $plugin ) : $plugin;
					?>
			<tr class="eos-dp-plugin-row eos-dp-post-row
						<?php
						echo 0 === $row ? ' fdp-row-1' : '';
						?>
						" data-row_id="<?php echo esc_attr( $plugin ); ?>">
			<td class="eos-dp-post-name-wrp">
				<span class="fdp-row-actions-ico dashicons dashicons-plus" title="<?php esc_attr__( 'Action buttons', 'eos-dp' ); ?>"></span>
				<span class="eos-dp-not-active-wrp"><input title="<?php printf( esc_attr__( 'Activate/deactivate all plugins when %s is disabled', 'eos-dp' ), esc_attr( $plugin_title ) ); ?>" class="eos-dp-global-chk-row" type="checkbox" /></span>

				<span class="eos-dp-title">
							<?php
							if ( 1 === $row ) {
								$this->resize_col();
							}
							echo '<span class="fdp-plugin-title">';
							echo esc_html( $plugin_title );
							echo '</span>';

							?>
					</span>

				<span class="eos-dp-x-space"></span>
				<div class="eos-dp-actions">
				<a class="eos-dp-invert-selection fdp-has-tooltip" href="#"><span class="dashicons"><span style="display:inline-block"><span class="fdp-invert-up"></span><span class="fdp-invert-down"></span></span></span>
					<div class="fdp-tooltip"><?php esc_html_e( 'Invert selection', 'eos-dp' ); ?></div>
				</a>
				<a class="eos-dp-copy fdp-has-tooltip" href="#"><span class="dashicons dashicons-admin-page"></span>
					<div class="fdp-tooltip"><?php esc_html_e( 'Copy this row settings', 'eos-dp' ); ?></div>
					<div class="fdp-action-msg fdp-msg-success" style="opacity:0;position:absolute;top:34px;background:#fff;padding:10px;transition:opacity 0.5s linear"><?php esc_html_e( 'Row settings copied', 'eos-dp' ); ?></div>
					<div class="fdp-action-msg fdp-msg-error" style="opacity:0;display:none;position:absolute;top:34px;background:#fff;padding:10px;transition:opacity 0.5s linear"><?php esc_html_e( 'It was not possible to copy the row settings', 'eos-dp' ); ?></div>
				</a>
				<a class="eos-dp-paste fdp-has-tooltip" href="#"><span class="dashicons dashicons-category"></span>
					<div class="fdp-tooltip"><?php esc_html_e( 'Paste last copied row settings', 'eos-dp' ); ?></div>
				</a>
						<?php do_action( 'eos_dp_action_buttons' ); ?>
				</div>
			</td>
						<?php
							$k = 0;
							foreach ( $this->active_plugins as $plugin_col ) {
									$active = $opt_array && in_array( $plugin_col, $opt_array ) ? false : true;
								if ( in_array( $plugin_col, array_keys( $this->plugins_by_dirs ) ) ) {
									?>
			<td class="center<?php echo $active ? ' eos-dp-active' : ''; echo $plugin === $plugin_col ? ' eos-no-events' : ''; ?>" data-path="<?php echo esc_attr( $plugin_col ); ?>">
				<div class="eos-dp-td-chk-wrp eos-dp-td-post-type-chk-wrp">
				<input class="eos-dp-row-<?php echo esc_attr( $row ); ?> eos-dp-col-<?php echo esc_attr( $k + 1 ); ?>" type="checkbox"<?php echo $active ? ' checked' : ''; ?> data-path="<?php echo esc_attr( $plugin_col ); ?>" />
				</div>
			</td>
									<?php
									++$k;
								}
							}
							?>
			</tr>
					<?php
			}		
				
			
			++$row;
		}
	}

	public function action_buttons( $page_slug ) {
		require_once EOS_DP_PLUGIN_DIR . '/admin/templates/partials/eos-dp-action-buttons.php';
	}

	public function get_nonces_map() {
		return array(
			sanitize_text_field( $this->page_slug )   => 'eos_dp_by_plugin_nonce'
		);
	}
}
