<?php
/**
 * Class for the Translations page.
 *
 * @package Freesoul Deactivate Plugins
 */

defined( 'ABSPATH' ) || exit; // Exit if accessed directly.

class FDP_Translation_Pages extends Eos_Fdp_Plugins_Manager_Page {

  public $section_id;
  public $button_class;
  private $url;

  public function before_section( $page_slug ){
    wp_nonce_field( 'eos_dp_pro_save_translation_urls','eos_dp_pro_save_translation_urls' );
    $this->button_class = 'fdp-translation-pages';
    $this->section_id = 'eos-dp-translation-pages-section';
    wp_enqueue_script( 'fdp-translation-urls',EOS_DP_PRO_PLUGIN_URL.'/admin/js/fdp-translation-urls.js',array( 'jquery','eos-dp-backend' ) );
  }
  public function section( $page_slug ){
    $opts = eos_dp_get_option( 'eos_dp_pro_main' );
    $value = isset( $opts['translation_urls'] ) ? $opts['translation_urls'] : '';
    ?>
    <style id="fdp-translation-pages-css"></style>
    <section id="<?php echo esc_attr( $this->section_id ); ?>" style="min-width:0 !important" class="<?php echo esc_attr( apply_filters( 'fdp_section_class_name','eos-dp-section' ) );echo ' '.esc_attr( $this->section_common_class ); ?>" data-page_slug="<?php echo esc_attr( $page_slug ); ?>">
      <h2><?php _e( 'Translation URLs.','eos-dp-pro' ); ?></h2>
  		<div class="eos-dp-explanation" style="margin-bottom:48px">
  			<p><?php _e( 'Write the URL fragments that should be ignored by FDP.','eos-dp-pro' ); ?></p>
  			<p><?php printf( __( 'If you write for example en, fr, then FDP will consider %s the same URL as %s and %s, and will deactivate the same plugins.','eos-dp-pro' ),get_home_url().'/sample-page',get_home_url().'/en/sample-page/',get_home_url().'/fr/sample-page/' ); ?></p>
  			<p><?php _e( 'Thanks to this feature you can use any kind of plugin for translations.','eos-dp-pro' ); ?></p>
  		</div>
      <?php do_action( 'eos_dp_before_wrapper' ); ?>
      <div id="eos-dp-wrp" style="max-width:900px">
        <label for="fdp-translation-urls-area"><?php _e( 'Write here the URL fragments that FDP should ignore. Separate them by comma.','eos-dp-pro' ); ?></label>
        <p><textarea id="fdp-translation-urls-search-area" class="widefat" style="width:100%" rows="20" placeholder="<?php esc_attr_e( 'E.g. en, fr, ?lank=en, ?lang=fr','eos-dp-pro' ); ?>"><?php echo esc_html( $value ); ?></textarea></p>
        <div id="fdp-progress-bar" style="height:16px;width:0;background:rgba(37,48,66,0.5)"></div>

        <div id="fdp-success" class="eos-dp-notice notice notice-success eos-dp-opts-msg_success eos-hidden" style="padding:10px"><?php _e( 'Changed saved.','eos-dp-pro' ); ?></div>
    		<div id="fdp-fail" class="eos-dp-notice notice notice-error eos-hidden" style="padding:10px" data-default_msg="<?php esc_attr_e( 'Something went wrong!','eos-dp-pro' ); ?>"></div>

      </div>
    </section>
    <?php
  }
  public function tableBody( $page_slug ){
    return;
  }

  public function get_nonces_map(){
    return;
  }

  public function footer( $button_class ){
    require_once EOS_DP_PLUGIN_DIR.'/admin/templates/partials/eos-dp-footer.php';
    if( current_user_can( 'activate_plugins' ) ){
      eos_dp_save_button( $button_class );
    }
  }
}
