<?php
/**
 * Code for the admin.
 *
 * @package Freesoul Deactivate Plugins
 */

defined( 'ABSPATH' ) || exit; // Exit if accessed directly.

$plugin = EOS_DP_PRO_PLUGIN_BASE_NAME;

if(
	isset( $_GET['page'] )
	&& in_array(
		$_GET['page'],array(
			'eos_dp_menu',
			'eos_dp_by_archive',
			'eos_dp_testing',
			'eos_dp_ajax',
			'eos_dp_admin',
			'eos_dp_admin_url',
			'eos_dp_rest_api',
			'eos_dp_by_post_requests',
			'eos_dp_report',
			'eos_dp_code_browser',
			'eos_dp_pro_general_bloat'
		)
	) ){
	add_action( 'admin_footer', 'eos_dp_pro_scripts',9999 ); //we enqueue the scripts for back-end,
}

/**
 * Enqueue scripts for backend
 */
function eos_dp_pro_scripts() {
	if( !defined( 'EOS_DP_PLUGIN_DIR' ) || !defined( 'EOS_DP_PRO_PLUGIN_URL' ) ) return;
	if( 'eos_dp_code_browser' === $_GET['page'] ){
		$params = array();
	  $params['codeEditor'] = wp_enqueue_code_editor( array( 'type'=>'text/css' ) );
	  wp_localize_script('jquery','fdp_code_mirror',$params);
	  wp_enqueue_script('wp-theme-plugin-editor');
	  wp_enqueue_script('fdp-code-browser',EOS_DP_PRO_PLUGIN_URL.'/admin/js/eos-dp-pro-code-browser.js',array( 'jquery','wp-theme-plugin-editor' ) );
	  wp_enqueue_style('wp-codemirror');
	}
	else{
		$deps = array( 'jquery' );
		if( isset( $_GET['eos_dp_ajax'] ) ){
			$deps[] = 'select2';
		}
		wp_enqueue_script( 'eos-dp-pro-backend',EOS_DP_PRO_PLUGIN_URL.'/admin/js/fdp-pro-backend-2.7.js',$deps,EOS_DP_PRO_VERSION );
		wp_localize_script( 'eos-dp-pro-backend','eos_dp_pro_js',array(
			'is_rtl' => is_rtl(),
			'page' => esc_js( esc_attr( $_GET['page'] ) ),
			'no_actions' => esc_js( esc_html( __( 'No actions detected','eos-dp-pro' ) ) ),
			'action_type' => isset( $_GET['page'] ) && 'eos_dp_by_post_requests' === $_GET['page'] ? 'post' : 'ajax',
			'labels_gpsi' => wp_json_encode( eos_dp_pro_gpsi_labels() ),
			'labels_gtmetrix' => wp_json_encode( eos_dp_pro_gtmetrix_labels() )
		) );
	}
}

//It adds the plugin setting page under plugins menu
function eos_dp_pro_options_page(){
	if( !defined( 'EOS_DP_PLUGIN_DIR' ) || !eos_dp_pro_can_settings( true ) ) return;
	$capability = current_user_can( 'fdp_plugins_viewer' ) ? 'read' : 'activate_plugins';
	$capability = apply_filters( 'eos_dp_settings_capability',$capability );
	$menu_pages = apply_filters( 'fdp_pro_menu_pages', array(
		array( __( 'Desktop','eos-dp-pro' ),__( 'Desktop','eos-dp-pro' ),$capability ,'eos_dp_desktop','eos_dp_desktop_callback',10 ),
		array( __( 'Error 404 page','eos-dp-pro' ),__( 'Error 404 page','eos-dp-pro' ),$capability ,'eos_dp_404','eos_dp_404_callback',10 ),
		array( __( 'Disabled on backend','eos-dp-pro' ),__( 'Disabled on backend','eos-dp-pro' ),$capability ,'eos_dp_backend_everywhere','eos_dp_backend_everywhere_callback',10 ),
		array( __( 'Deactivate depending on Ajax actions','eos-dp-pro' ),__( 'Deactivate depending on Ajax actions','eos-dp-pro' ),$capability ,'eos_dp_ajax','eos_dp_ajax_callback',10 ),
		array( __( 'Deactivate depending on Post requests','eos-dp-pro' ),__( 'Deactivate depending on Post requests','eos-dp-pro' ),$capability ,'eos_dp_by_post_requests','eos_dp_by_post_request_callback',10 ),
		array( __( 'Testing','eos-dp-pro' ),__( 'Testing','eos-dp-pro' ),$capability ,'eos_dp_testing','eos_dp_pro_testing_callback',20 ),
		array( __( 'Status','eos-dp-pro' ),__( 'Status','eos-dp-pro' ),$capability ,'eos_dp_status','eos_dp_pro_status_callback',30 ),
		array( __( 'Reports','eos-dp-pro' ),__( 'Reports','eos-dp-pro' ),$capability ,'eos_dp_report','eos_dp_pro_report_callback',40 ),
		array( __( 'Bulk Actions','eos-dp-pro' ),__( 'Bulk Actions','eos-dp-pro' ),$capability ,'eos_dp_pro_bulk_actions','eos_dp_pro_bulk_actions_callback',80 ),
		array( __( 'Main Settings','eos-dp-pro' ),__( 'Main Settings','eos-dp-pro' ),$capability ,'eos_dp_pro_settings','eos_dp_pro_settings_callback',90 ),
		array( __( 'Plugin Settings','eos-dp-pro' ),__( 'Plugin Settings','eos-dp-pro' ),$capability ,'eos_dp_pro_plugins','eos_dp_pro_plugins_callback',100 ),
		array( __( 'Removed Hooks','eos-dp-pro' ),__( 'Removed Hooks','eos-dp-pro' ),$capability ,'eos_dp_pro_removed_hooks','eos_dp_pro_removed_hooks_callback',110 ),
		array( __( 'Assets','eos-dp-pro' ),__( 'Assets','eos-dp-pro' ),$capability ,'eos_dp_pro_assets','eos_dp_pro_assets_callback',120 ),
		array( __( 'General Bloat','eos-dp-pro' ),__( 'General Bloat','eos-dp-pro' ),$capability ,'eos_dp_pro_general_bloat','eos_dp_pro_general_bloat_callback',130 ),
		array( __( 'Autolad','eos-dp-pro' ),__( 'Autolad','eos-dp-pro' ),$capability ,'eos_dp_pro_autoload','eos_dp_pro_autoload_callback',130 ),
		array( __( 'Code Browser','eos-dp-pro' ),__( 'Code Browser','eos-dp-pro' ),$capability ,'eos_dp_code_browser','eos_dp_pro_code_browser_callback',140 ),
		array( __( 'Users','eos-dp-pro' ),__( 'Users','eos-dp-pro' ),$capability,'eos_dp_logged','eos_dp_pro_logged_callback',150 ),
		array( __( 'Unlogged Users','eos-dp-pro' ),__( 'Unlogged Users','eos-dp-pro' ),$capability,'eos_dp_unlogged','eos_dp_pro_unlogged_callback',160 ),
		array( __( 'Hooks Recorder','eos-dp-pro' ),__( 'Hooks Recorder','eos-dp-pro' ),$capability ,'eos_dp_pro_hooks_recorder','eos_dp_pro_hooks_recorder_callback',170 ),
		array( __( 'Whois','eos-dp-pro' ),__( 'Whois','eos-dp-pro' ),$capability ,'eos_dp_pro_whois','eos_dp_pro_whois_callback',180 ),
		array( __( 'Translation Pages','eos-dp-pro' ),__( 'Translation Pages','eos-dp-pro' ),$capability ,'eos_dp_translation_urls','eos_dp_translation_urls_callback',190 ),
		
		array( __( 'Active on Frontend','eos-dp-pro' ),__( 'Active on Frontend','eos-dp-pro' ),$capability ,'eos_dp_frontend_untouchables','eos_dp_pro_frontend_untouchables_callback',210 ),
		array( __( 'Active on Backend','eos-dp-pro' ),__( 'Active on Backend','eos-dp-pro' ),$capability ,'eos_dp_backend_untouchables','eos_dp_pro_backend_untouchables_callback',210 ),
		array( __( 'Disabled on Frontend','eos-dp-pro' ),__( 'Disabled on Frontend','eos-dp-pro' ),$capability ,'eos_dp_frontend_everywhere','eos_dp_pro_frontend_everywhere_callback',220 ),
		array( __( 'Cron Jobs','eos-dp-pro' ),__( 'Cron Jobs','eos-dp-pro' ),$capability ,'eos_dp_cron','eos_dp_pro_cron_callback',220 ),
		array( __( 'Rest API','eos-dp-pro' ),__( 'Rest API','eos-dp-pro' ),$capability ,'eos_dp_rest_api','eos_dp_pro_rest_api_callback',220 ),
		array( __( 'By Plugin','eos-dp-pro' ),__( 'By Plugin','eos-dp-pro' ),$capability ,'eos_dp_by_plugin','eos_dp_by_plugin_callback',220 ),
		array( __( 'Plugin Conflicts','eos-dp-pro' ),__( 'Plugin Conflicts','eos-dp-pro' ),$capability ,'eos_dp_conflicts','eos_dp_conflicts_callback',220 ),
	) );
	foreach( $menu_pages as $menu_page ) {
		$file = EOS_DP_PRO_PLUGIN_DIR.'/admin/templates/pages/fdp-' . str_replace( array( 'eos_dp_pro_', 'eos_dp_', '_' ), array( '', '', '-' ), $menu_page[3] ) . '.php';
		if( isset( $_GET['page'] ) &&  $menu_page[3] === sanitize_text_field( $_GET['page' ] ) && file_exists( $file ) ) {
			require_once  sanitize_text_field( $file );
		}
		add_submenu_page( 
			'fdp_hidden_menu',
			esc_attr( $menu_page[0] ),
			esc_html( $menu_page[1] ),
			esc_html( $menu_page[2] ),
			esc_attr( $menu_page[3] ),
			esc_attr( isset( $_GET['page'] ) && $menu_page[3] === sanitize_text_field( $_GET['page'] ) ? $menu_page[4] : '__return_false' ),
			absint( $menu_page[5] ) 
		);
	}
	if( isset( $_GET['page'] ) &&  'eos_dp_pro_license' === sanitize_text_field( $_GET['page' ] ) ) {
		require_once  EOS_DP_PRO_PLUGIN_DIR.'/admin/templates/pages/fdp-license.php';
	}

	add_submenu_page(
		'eos_dp_menu',
		__( 'License','eos-dp-pro' ),
		__( 'License','eos-dp-pro' ),
		$capability ,
		'eos_dp_pro_license',
		esc_attr( isset( $_GET['page'] ) && 'eos_dp_pro_license' === sanitize_text_field( $_GET['page'] ) ? 'eos_dp_pro_license_callback' : '__return_false' ),
		200 
	);

	if( isset( $_GET['page'] ) && 'eos_dp_pro_import_export' === sanitize_text_field( $_GET['page'] ) ) {
		require_once EOS_DP_PRO_PLUGIN_DIR.'/admin/templates/pages/fdp-import-export.php';
		$callback = 'eos_dp_pro_import_export_callback';
	}
	else{
		$callback = '__return_false';
	}

	add_management_page( 
		esc_html__( 'FDP Import/Export','eos-dp-pro' ),
		esc_html__( 'FDP Import/Export','eos-dp-pro' ),
		$capability ,
		'eos_dp_pro_import_export',
		sanitize_key( $callback ),
		70
	);
}

add_action( 'admin_menu','eos_dp_pro_options_page',9999 );
add_filter( 'admin_body_class', 'eos_dp_pro_body_class' );
//It adds a CSS class to the body for admin pages
function eos_dp_pro_body_class( $classes ) {
    return $classes.' eos-dp-pro';
}
add_filter( "plugin_action_links_$plugin", 'eos_dp_pro_plugin_add_settings_link' );
//It adds a settings link to the action links in the plugins page
function eos_dp_pro_plugin_add_settings_link( $links ) {
	$page = defined( 'EOS_DP_VERSION' ) ? 'eos_dp_menu&pointer=true' : 'eos_dp_pro_installations';
	$settings_link = '<a href="'.add_query_arg( 'page',$page,admin_url( 'admin.php' ) ).'">' . esc_html__( 'Settings','eos-dp-pro' ). '</a>';
  	array_push( $links, $settings_link );
	$main_opts = eos_dp_get_option( 'eos_dp_pro_main' );
	if( !defined( 'FDP_PRO_LICENSE_EDD' ) || !FDP_PRO_LICENSE_EDD ){
		$licenseA = isset( $main_opts['eos_dp_license'] ) ? $main_opts['eos_dp_license'] : false;
		if( !$licenseA || !isset( $licenseA['fdp-license-key'] ) || '' === $licenseA['fdp-license-key'] ){
			$activation_link = '<a href="'.add_query_arg( 'page','eos_dp_pro_license',admin_url( 'admin.php' ) ).'"><b>' . esc_html__( 'License activation','eos-dp-pro' ). '</b></a>';
		  	array_push( $links, $activation_link );
		}
	}
	else{
		$license_key = eos_dp_get_option( 'eos_dp_pro_edd_license_key' );
		if( !$license_key || '' === $license_key ){
			$activation_link = '<a href="'.add_query_arg( 'page','eos_dp_pro_license',admin_url( 'admin.php' ) ).'"><b>' . esc_html__( 'License activation','eos-dp-pro' ). '</b></a>';
		 	array_push( $links, $activation_link );
		}
	}
	$links[] = '<a href="https://support.freesoul-deactivate-plugins.com/" target="_fdp_premium_support" rel="noopener">' . esc_html( 'Support', 'eos-dp-pro' ) . '</a>';
  return $links;
}

add_filter( 'eos_dp_deactivation_pages','eos_dp_pro_deactivation_pages' );
//Filter array of deactivation setting Pages
function eos_dp_pro_deactivation_pages( $pages ){
	$pages[] = 'eos_dp_logged';
	$pages[] = 'eos_dp_cron';
	$pages[] = 'eos_dp_rest_api';
	return $pages;
}
add_action( 'eos_dp_action_buttons','eos_dp_pro_action_buttons',20 );
//It adds premium action buttons
function eos_dp_pro_action_buttons(){
	$dir = is_rtl() ? 'right' : 'left';
	if( isset( $_GET['page'] ) && in_array( $_GET['page'],array( 'eos_dp_menu','eos_dp_by_archive','eos_dp_by_term_archive' ) ) && !isset( $_GET['eos_dp_home'] ) ){
	?>
	<a class="eos-dp-paste-post-types fdp-has-tooltip" href="#"><span class="dashicons dashicons-category"></span><span style="margin-top:-10px;margin-<?php echo $dir; ?>:-22px" class="dashicons">PT</span>
		<div class="fdp-tooltip"><?php _e( 'Get active plugins from the post type settings','eos-dp' ); ?></div>
	</a>
	<a href="#" class="eos-dp-pro-autosettings fdp-has-tooltip"><span class="dashicons dashicons-plugins-checked"></span>
		<div class="fdp-tooltip"><?php _e( 'Suggest unused plugins for this row','eos-dp' ); ?></div>
	</a>
	<a href="#" class="eos-dp-pro-autosettings-all-from-row fdp-has-tooltip"><span class="dashicons dashicons-plugins-checked"></span><span style="margin-top:-10px;margin-<?php echo $dir; ?>:-22px" class="dashicons dashicons-admin-page"></span>
		<div class="fdp-tooltip"><?php _e( 'Suggest unused plugins for all rows starting from this row','eos-dp' ); ?></div>
	</a>
	<?php if( defined( 'FDP_PRO_CHECK_ERRORS' ) && FDP_PRO_CHECK_ERRORS  ){ ?>
	<a href="#" class="eos-dp-pro-errors-check fdp-has-tooltip"><span class="eos-dp-error-icon"></span>
		<div class="fdp-tooltip"><?php _e( 'Check errors','eos-dp' ); ?></div>
	</a>
	<?php
	}
	if( 'eos_dp_admin' !== $_GET['page'] ){ ?>
		<a href="#" class="eos-dp-pro-gtmetrix fdp-has-tooltip fdp-left-tooltip" style="position:relative">
			<span class="eos-dp-gtmetrix-icon"></span>
			<span style="font-size:20px;margin-top:-10px;margin-<?php echo $dir; ?>:-10px" class="dashicons dashicons-desktop"></span>
			<div class="fdp-tooltip"><?php _e(  'GTMetrix test','eos-dp' ); ?></div>
		</a>
		<a href="#" class="eos-dp-pro-gpsi fdp-has-tooltip fdp-left-tooltip " style="position:relative" data-device="mobile">
			<span class="eos-dp-gpsi-icon"></span>
			<span style="margin-top:0;margin-<?php echo $dir; ?>:-10px" class="dashicons dashicons-smartphone"></span>
			<div class="fdp-tooltip"><?php _e( 'Google PSI test on mobile','eos-dp' ); ?></div>
		</a>
		<a href="#" class="eos-dp-pro-gpsi fdp-has-tooltip fdp-left-tooltip" style="position:relative" data-device="desktop">
			<span class="eos-dp-gpsi-icon"></span>
			<span style="font-size:20px;margin-top:-10px;margin-<?php echo $dir; ?>:-10px" class="dashicons dashicons-desktop"></span>
			<div class="fdp-tooltip"><?php _e(  'Google PSI test on desktop','eos-dp' ); ?></div>
		</a>
		<?php
		}
	}
}

//It returns the GTMetrix test locations
function eos_dp_pro_gtmetrix_locations(){
	return array(
		__( 'Default','eos-dp-pro' ),
		'Vancouver, Canada',
		'London, UK',
		'Sydney, Australia',
		'Dallas, USA',
		'Mumbai, India',
		'São Paulo, Brazil',
		'Hong Kong, China'
	);
}


function eos_dp_pro_nofdp_callback(){
	?>
	<section>
		<div id="eos-dp-installation">
		<?php
		if( !defined( 'EOS_DP_VERSION' ) ){
			die( eos_dp_pro_dependency_message() );
		}
		?>
		</div>
	</section>
	<?php
}

add_action( 'admin_init','eos_dp_pro_initialization' );
//It redirects to the plugin settings page on successfully plugin activation
function eos_dp_pro_initialization(){
	if( get_transient( 'fdp-pro-notice-succ' ) ){
		delete_transient( 'fdp-pro-notice-succ' );
		if( !get_transient( 'fdp-pro-updating-mu' ) && defined( 'EOS_DP_PRO_VERSION' ) ){
			wp_safe_redirect( admin_url( 'plugins.php?page=eos_dp_by_post_type' ) );
		}
	}
}
//Return a message depending on the presence of FDP
function eos_dp_pro_dependency_message(){
	$slug = 'freesoul-deactivate-plugins';
	if( !is_dir( WP_PLUGIN_DIR . '/freesoul-deactivate-plugins' ) ){
		$action = 'install-plugin';
		$url = wp_nonce_url(
			add_query_arg(
				array(
					'action' => $action,
					'plugin' => $slug
				),
				admin_url( 'update.php' )
			),
			'install-plugin_'.$slug
		);
		$message = '<p><h2>'.esc_html__( 'FDP Pro needs Freesoul Deactivate Plugins','eos-dp-pro' ).'</h2></p>';
		$message .= '<p><a class="eos-dp-pro-install" style="padding: 4px 8px;position: relative;top: -3px;text-decoration: none;border: none;border: 1px solid #ccc;border-radius: 2px;background: #f7f7f7;text-shadow: none;font-weight: 600;font-size: 13px;line-height: normal;color: #0073aa;cursor: pointer" href="'.esc_url( $url ) . '">'.esc_html__( 'Install Freesoul Deactivate Plugins','eos-dp-pro' ).'</a></p>';
	}
	else{
		$plugin = 'freesoul-deactivate-plugins/freesoul-deactivate-plugins.php';
		$action = 'activate';
		$url = sprintf( admin_url( 'plugins.php?action='.$action.'&plugin=%s&plugin_status=all&paged=1&s' ),$plugin );
		$url = wp_nonce_url( $url,$action.'-plugin_'.$plugin );
		$message = '<p><h2>'.esc_html__( 'FDP Pro needs Freesoul Deactivate Plugins','eos-dp-pro' ).'</h2>';
		$message .= '<p><a class="eos-dp-pro-install" style="padding: 4px 8px;position: relative;top: -3px;text-decoration: none;border: none;border: 1px solid #ccc;border-radius: 2px;background: #f7f7f7;text-shadow: none;font-weight: 600;font-size: 13px;line-height: normal;color: #0073aa;cursor: pointer" href="'.esc_url( $url ) . '">'.esc_html__( 'Activate Freesoul Deactivate Plugins','eos-dp-pro' ).'</a></p>';
	}
	$ajaxurl = admin_url( 'admin-ajax.php' );
	$message .= '<script>
	var eos_dp_install_button = document.getElementsByClassName("eos-dp-pro-install")[0];
	if(eos_dp_install_button){
		eos_dp_install_button.addEventListener("click",function(e){
			e.stopPropagation();
			e.preventDefault();
			window.top.location.href = e.target.href;
			return false;
		});
	}
	</script>';
	return $message;
}

if( !isset( $_GET['eos_dp_home'] ) ){
	add_action( 'fdp_before_row_filters','eos_dp_pro_after_singles_title' );
}
//Add autosuggestions for all pages
function eos_dp_pro_after_singles_title(){
	?>
	<div class="eos-dp-margin-top-15">
		<span id="eos-dp-autosuggest-all" class="button"><?php esc_html_e( 'Suggest used plugins','eos-dp-pro' ); ?></span>
		<span id="eos-dp-stop-process" class="eos-dp-not-active button" title="<?php esc_attr_e( 'Stop autosuggestion','eos-dp-pro' ); ?>"><?php esc_html_e( 'Stop','eos-dp-pro' ); ?></span>
	</div>
	<?php
}

add_action( 'fdp_table_head_first_col_eos_dp_admin','eos_dp_pro_before_wrapper_actions' );
add_action( 'fdp_table_head_first_col_eos_dp_by_archive','eos_dp_pro_before_wrapper_actions' );
add_action( 'fdp_table_head_first_col_eos_dp_by_term_archive','eos_dp_pro_before_wrapper_actions' );
//Actions before wrapper
function eos_dp_pro_before_wrapper_actions(){
	?>
	<div class="eos-dp-mb-32">
		<span id="eos-dp-autosuggest-all" class="button"><?php _e( 'Suggest unused plugins','eos-dp' ); ?></span>
		<span id="eos-dp-stop-process" class="eos-dp-not-active button" title="<?php esc_attr_e( 'The process involving the current row will finish in any case','eos-dp-pro' ); ?>"><?php esc_html_e( 'Stop','eos-dp-pro' ); ?></span>
	</div>
<?php
}

add_action( 'eos_dp_backend_actions','eos_dp_pro_backend_actions' );
//Add PRO action buttons to backend settings page
function eos_dp_pro_backend_actions(){
	$dir = is_rtl() ? 'right' : 'left';
	?>
	<a href="#" class="eos-dp-pro-autosettings-all-from-row fdp-has-tooltip">
		<span class="dashicons dashicons-plugins-checked"></span>
		<span style="margin-top:-10px;margin-<?php echo $dir; ?>:-22px" class="dashicons dashicons-admin-page"></span>
		<div class="fdp-tooltip"><?php _e( 'Suggest unused plugins for all rows starting from this row','eos-dp' ); ?></div>
	</a>
	<?php
}

add_action( 'eos_dp_table_head_col_after','eos_dp_pro_table_col_after_callback' );
//Add actions on plugin column
function eos_dp_pro_table_col_after_callback(){
	if( !isset( $_GET['page'] ) || !in_array( $_GET['page'],array( 'eos_dp_menu','eos_dp_admin' ) ) ) return;
	?>
	<div class="eos-dp-pro-actions-col">
		<a href="#" class="eos-dp-pro-autosettings-col" title="<?php esc_attr_e( 'Suggest where I can disable this plugin','eos-dp-pro' ); ?>">
			<span class="dashicons dashicons-plugins-checked"></span>
		</a>
	</div>
	<?php
}

add_action( 'eos_dp_after_archive_title','eos_dp_pro_after_archive_title' );
add_action( 'eos_dp_after_terms_archive_title','eos_dp_pro_after_archive_title' );
//Actions after archive title
function eos_dp_pro_after_archive_title(){
	wp_nonce_field( 'eos_dp_pro_auto_settings', 'eos_dp_pro_auto_settings' );
}

add_action( 'eos_dp_after_table_bulk_actions','eos_dp_pro_storage_buttons' );
function eos_dp_pro_storage_buttons(){
	if( !isset( $_GET['page'] ) ) return;
	$id_from_get = implode( '_',$_GET );
	$sets = array( 'mem1' => 'Save Set 1','mem2' => 'Save Set 2','mem3' => 'Save Set 3','mem4' => 'Save Set 4' );
	?>
	<div style="display:inline-block"><span id="fdp-toggle-storage" class="hover dashicons dashicons-database" style="font-size: 30px;margin:0 5px"></span></div>
	<div id="eos-dp-storage-wrp" class="eos-hidden">
		<div id="eos-dp-storage-btns-set" style="margin-top:16px;display:inline-block">
		<?php foreach( $sets as $k => $v ){ ?>
		<span data-id="<?php echo esc_attr( $id_from_get ).'_'.$k; ?>" title="<?php printf( esc_attr__( 'Store actual options as %s','eos-dp-pro' ),$v ); ?>" id="eos-dp-<?php echo $k; ?>" class="button"><?php echo $v; ?></span>
		<?php } ?>
		</div>
		<div id="eos-dp-storage-btns-get" style="display:inline-block;margin:0 20px">
		<?php $n = 0;foreach( array( 'mem1' => 'Restore Set 1','mem2' => 'Restore Set 2','mem3' => 'Restore Set 3','mem4' => 'Restore Set 4' ) as $k => $v ){ ?>
		<span data-id="<?php echo esc_attr( $id_from_get ).'_'.$k; ?>" title="<?php printf( esc_attr__( 'Fill the table with the options stored as %s','eos-dp-pro' ),$sets[$k] ); ?>" id="eos-dp-get-<?php echo $n; ?>" class="button"><?php echo $v; ?></span>
		<?php ++$n; } ?>
		</div>
	</div>
	<script>var eos_dp_storage_page_id = "<?php echo esc_js( esc_attr( $id_from_get ) ); ?>";</script>
	<?php
}

add_action( 'wp_after_insert_post', 'eos_dp_pro_check_issues_after_saving' );
//Check if the disabled plugin give issues
function eos_dp_pro_check_issues_after_saving( $post_id ){
	$permalink = get_permalink( $post_id );
	if( !$permalink ) return;
	$clean_permalinkA = explode( '?',$permalink );
	$post_type = get_post_type( $post_id );
	if( $post_type ){
		$post_type_object = get_post_type_object( $post_type );
		if( $post_type_object && ( ( isset( $post_type_object->public ) && false === $post_type_object->public ) ) ){
			return;
		}
	}
	if( rtrim( get_home_url(),'/' ) === rtrim( $clean_permalinkA[0],'/' ) && absint( $post_id ) !== absint( get_site_option( 'page_on_front' ) ) ){
		return;
	}
	delete_site_transient( '_fdp_pro_post_nsg_'.$post_id );
	$opts = eos_dp_pro_get_option( 'eos_dp_pro_main' );
	if( !isset( $opts['eos_dp_general_setts'] ) ) return;
	$general_setts = $opts['eos_dp_general_setts'];
	if( !isset( $general_setts['after_saving_post'] ) || 'check' !== $general_setts['after_saving_post'] ) return;
	if( !isset( $general_setts['after_saving_notification'] ) || 'no_action' === $general_setts['after_saving_notification'] ) return;
	if( !eos_dp_pro_is_disabling_ok( $post_id,$permalink ) ){
		$msg = wp_kses(
			sprintf(
				__( 'It looks %s has some plugins that should not be disabled.','eos-dp-pro' ),
				'<a href="'.get_the_permalink( $post_id ).'" target="_blank">'.get_the_title( $post_id ).'</a>'
			),
			array( 'a' => array( 'href' => array(),'target' => array() ) )
		);
		set_site_transient( '_fdp_pro_post_nsg_'.$post_id,$msg,300 );
		if( 'backend_notice_and_email' === $general_setts['after_saving_notification'] ){
			$admin_email = isset( $general_setts['notifications_email'] ) && sanitize_email( $general_setts['notifications_email'] ) === $general_setts['notifications_email'] ? $general_setts['notifications_email'] : get_site_option( 'admin_email' );
			$msg = '<p>'.$msg.'</p>';
			$msg .= '<p>'.eos_dp_pro_unsubscribe_msg().'</p>';
			$msg = wp_kses( $msg,array( 'p' => array(),'strong' => array(),'a' => array( 'href' => array(),'rel' => array() ) ) );
			eos_dp_pro_sendmail( sanitize_email( $admin_email ),sanitize_email( $admin_email ),sprintf( esc_html__( 'Maybe wrong plugins were disabled on %s.','eos-dp-pro' ),get_bloginfo( 'name ' ) ),$msg,array(),$admin_email );
		}
	}

}

function eos_dp_pro_is_disabling_ok( $post_id,$permalink ){
	$args = array(
		'eos_dp_debug' => 'no_errors',
		'eos_dp_preview' => 'true',
		'eos_dp_offset' => '',
		'after_save' => 'true',
		'fdp_post_id' => $post_id,
		'eos_dp_pro_id' => md5( EOS_DP_PRO_TESTING_UNIQUE_ID ),
		'test_id' => 10000*microtime(1),
		'site_in_progress' => substr( md5( esc_attr( get_option( 'comingsoon_input_psw_email' ) ).'_' ),0,8 )
	);
	$transient_name = 'fdp_test_after_save_'.$post_id;
	set_transient( $transient_name,'',600 );
	$url = str_replace( '&amp;','&',add_query_arg( $args,wp_nonce_url( $permalink,'eos_dp_preview','eos_dp_preview' ) ) );
	require_once EOS_DP_PLUGIN_DIR. '/admin/eos-dp-ajax.php';
	$body_all_plugins_Count = eos_dp_pro_count_by_url( $url,'all' );
	$args['test_id'] = 10000*microtime(1);
	$post_meta = get_post_meta( $post_id,'_eos_deactive_plugins_key',true );
	$plugins = explode( ',',$post_meta );
	if( is_array( $plugins ) ){
		$plugins = array_unique( $plugins );
		if( in_array( 'freesoul-deactivate-plugins/freesoul-deactivate-plugins.php',$plugins ) ){
			unset( $plugins[array_search( 'freesoul-deactivate-plugins/freesoul-deactivate-plugins.php',$plugins )] );
		}
	}
	$paths_var = ';pn:'.implode( ';pn:',$plugins );
	set_transient( $transient_name,$paths_var,600 );
	$url = str_replace( '&amp;','&',add_query_arg( $args,wp_nonce_url( $permalink,'eos_dp_preview','eos_dp_preview' ) ) );
	$body_disabled_plugins_Count = eos_dp_pro_count_by_url( $url,'disabled_plugins' );
	delete_transient( $transient_name );
	return absint( $body_all_plugins_Count ) > 0 && $body_all_plugins_Count === $body_disabled_plugins_Count;
}

add_action( 'activated_plugin','eos_dp_pro_after_plugins_activation' );
//Actions after a third plugin is activated_plugin
function eos_dp_pro_after_plugins_activation( $plugin ){
	set_site_transient( 'fdp_plugin_activated_notification',sanitize_text_field( $plugin ),60*60*24 );
	$opts = eos_dp_pro_get_option( 'eos_dp_pro_main' );
	if( !isset( $opts['eos_dp_general_setts'] ) ) return;
	$general_setts = $opts['eos_dp_general_setts'];
	if( isset( $general_setts['globally_disabled_plugins'] ) && is_array( $general_setts['globally_disabled_plugins'] ) && in_array( $plugin,$general_setts['globally_disabled_plugins'] ) ){
	 return;
 	}
	$settings_page = '<a href="'.admin_url( 'admin.php?page=eos_dp_pro_settings' ).'" rel="noopener">'.__( 'the Settings','eos-dp-pro' ).'</a>';
	$message = '<p>'.__( 'Hello,','eos-dp-pro' ).'</p>';
	if( isset( $general_setts['default_plugin_activation'] ) && 'deactivate_everywhere' === $general_setts['default_plugin_activation'] ){
		//let's deactivate it everywhere in the FDP settings
		require_once EOS_DP_PRO_PLUGIN_DIR.'/admin/eos-dp-pro-ajax.php';
		$option_keys = array_keys( eos_dp_pro_options_keys( true ) );
		$singles_post_types = array_keys( eos_dp_plugins_table() );
		if( isset( $singles_post_types['attachment'] ) ){
			unset( $singles_post_types['attachment'] );
		}
		foreach( $option_keys as $option_key ){
			if( in_array( $option_key,array( 'eos_dp_mobile','eos_dp_desktop' ) ) ) continue;
			$post_types = 'eos_dp_singles' === $option_key ? $singles_post_types : array();
			eos_dp_pro_execute_bulk_action( $plugin,'deactivate',$option_key,$post_types );
		}
		$message .= '<p>'.sprintf( __( 'According to %s, FDP has deactivated it everywhere.','eos-dp-pro' ),$settings_page ).'</p>';
	}
	else{
		$message .= '<p>'.sprintf( __( 'According to %s, FDP has kept it active everywhere, but the user may have changed the FDP settings.','eos-dp-pro' ),$settings_page ).'</p>';
	}
	if( isset( $general_setts['notification_after_plugin_activation'] ) && in_array( $general_setts['notification_after_plugin_activation'],array( 'send_email','send_email_name_user' ) ) ){
		$user = wp_get_current_user();
		$plugin_name = ucwords( str_replace( '-',' ',dirname( $plugin ) ) );
		$user_activator = isset( $general_setts['notification_after_plugin_activation'] ) && 'send_email_name_user' === $general_setts['notification_after_plugin_activation'] ? $user->user_login : __( 'Someone','eos-dp-pro' );
		$message .= '<p>'.sprintf( __( '%s has right now activated %s.','eos-dp-pro' ),$user_activator,$plugin_name ).'</p>';

		$unsubscribe_link =
			wp_nonce_url(
				get_home_url(),
				'_fdp_unsubscribe_notification',
				'_fdp_unsubscribe_notification'
			);
		$admin_email = isset( $general_setts['notifications_email'] ) && sanitize_email( $general_setts['notifications_email'] ) === $general_setts['notifications_email'] ? $general_setts['notifications_email'] : get_site_option( 'admin_email' );
		$message .= '<p>'.eos_dp_pro_unsubscribe_msg().'</p>';
		$message = wp_kses( $message,array( 'p' => array(),'strong' => array(),'a' => array( 'href' => array(),'rel' => array() ) ) );
		eos_dp_pro_sendmail( sanitize_email( $admin_email ),sanitize_email( $admin_email ),sprintf( esc_html__( 'The plugin %s has been activated on %s','eos-dp-pro' ),$plugin_name,get_site_url() ),$message,array() );
	}
}


add_action( 'deactivated_plugin','eos_dp_pro_after_plugins_deactivation' );
//Actions after a third plugin is deactivated_plugin
function eos_dp_pro_after_plugins_deactivation( $plugin ){
	if( !isset( $plugin ) || !$plugin ) return;
	$opts = eos_dp_pro_get_option( 'eos_dp_pro_main' );
	if( !isset( $opts['eos_dp_general_setts'] ) ) return;
	$general_setts = $opts['eos_dp_general_setts'];
	if( !isset( $opts['globally_disabled_plugins'] ) ){
		$general_setts['globally_disabled_plugins'] = array( sanitize_text_field( $plugin ) );
	}
	else{
		if( !in_array( $plugin,$opts['globally_disabled_plugins'] ) ){
			$general_setts['globally_disabled_plugins'][] = sanitize_text_field( $plugin );
		}
	}
	$opts['eos_dp_general_setts'] = $general_setts;
	eos_dp_pro_update_option( 'eos_dp_pro_main',$opts );
	if( isset( $general_setts['notification_after_plugin_deactivation'] ) && in_array( $general_setts['notification_after_plugin_deactivation'],array( 'send_email','send_email_name_user' ) ) ){
		$user = wp_get_current_user();
		$plugin_name = ucwords( str_replace( '-',' ',dirname( $plugin ) ) );
		$message = '<p>'.__( 'Hello,','eos-dp-pro' ).'</p>';
		$user_deactivator = isset( $general_setts['notification_after_plugin_deactivation'] ) && 'send_email_name_user' === $general_setts['notification_after_plugin_deactivation'] ? $user->user_login : __( 'Someone','eos-dp-pro' );
		$message .= '<p>'.sprintf( __( '%s has right now deactivated %s.','eos-dp-pro' ),$user_deactivator,$plugin_name ).'</p>';
		$admin_email = isset( $general_setts['notifications_email'] ) && sanitize_email( $general_setts['notifications_email'] ) === $general_setts['notifications_email'] ? $general_setts['notifications_email'] : get_site_option( 'admin_email' );
		$unsubscribe_link =
			wp_nonce_url(
				get_home_url(),
				'_fdp_unsubscribe_notification',
				'_fdp_unsubscribe_notification'
			);
		$message .= '<p>'.sprintf( __( "If you don't want to receive anymore this kind of notification, click %s. (This link will expire in 24 hours).","eos-dp-pro" ),'<a href="'.$unsubscribe_link.'" rel="noopener">'.__( 'here','eos-dp-pro' ).'</a>' ).'</p>';
		$message = wp_kses( $message,array( 'p' => array(),'strong' => array(),'a' => array( 'href' => array(),'rel' => array() ) ) );
		eos_dp_pro_sendmail( sanitize_email( $admin_email ),sanitize_email( $admin_email ),sprintf( esc_html__( 'The plugin %s has been deactivated on %s','eos-dp-pro' ),$plugin_name,get_site_url() ),$message,array(),$admin_email );
	}
}

add_action( 'deleted_plugin','eos_dp_pro_after_plugins_deletion' );
//Actions after third plugin is deleted
function eos_dp_pro_after_plugins_deletion( $plugin ){
	if( !isset( $plugin ) || !$plugin ) return;
	$opts = eos_dp_pro_get_option( 'eos_dp_pro_main' );
	if( !isset( $opts['eos_dp_general_setts'] ) ) return;
	$general_setts = $opts['eos_dp_general_setts'];
	if( isset( $general_setts['globally_disabled_plugins'] ) && is_array( $general_setts['globally_disabled_plugins'] ) && in_array( $plugin,$general_setts['globally_disabled_plugins'] ) ){
		$k = array_search( $plugin,$general_setts['globally_disabled_plugins'] );
		unset( $general_setts['globally_disabled_plugins'][$k] );
		$opts['eos_dp_general_setts'] = $general_setts;
		eos_dp_pro_update_option( 'eos_dp_pro_main',$opts );
	}
}
//Send mail function
function eos_dp_pro_sendmail( $from,$to,$subject,$message,$attachments = array(),$reply_to = false ) {
	$headers = array( 'From: "'.get_bloginfo( 'name' ).'" <'.sanitize_email( $from ).'>' );
	$headers[] = 'Content-type: text/html';
	if( $reply_to ){
		$headers[] = 'Reply-To: <'.sanitize_email( $reply_to ).'>';
	}
	return wp_mail( $to, str_replace( '&#039;','\'',esc_html( $subject ) ), str_replace( '&#039;','\'',wp_kses( $message,array( 'p' => array(),'strong' => array(),'a' => array( 'href' => array(),'rel' => array() ) ) ) ),$headers,$attachments ) ? 1 : 0;
}

//Return text to unsubscribe from notifications_email
function eos_dp_pro_unsubscribe_msg(){
	$unsubscribe_link = wp_nonce_url(
		get_home_url(),
		'_fdp_unsubscribe_notification',
		'_fdp_unsubscribe_notification'
	);
	return sprintf( __( "If you don't want to receive anymore this kind of notification, click %s. (This link will expire in 24 hours).","eos-dp-pro" ),'<a href="'.$unsubscribe_link.'" rel="noopener">'.__( 'here','eos-dp-pro' ).'</a>' );
}
if( isset( $_REQUEST['export_file'] ) && 'true' === $_REQUEST['export_file'] ){
	eos_dp_pro_inline_style_no_menu( 'all' );
}
if( isset( $_REQUEST['page'] ) && 'eos_dp_report' === $_REQUEST['page'] && !isset( $_REQUEST['export_file'] ) ){
	eos_dp_pro_inline_style_no_menu( 'print' );
}
if(
	( isset( $_REQUEST['page'] ) && 'eos_dp_report' === $_REQUEST['page'] && isset( $_REQUEST['export_file'] ) && 'true' === $_REQUEST['export_file'] )
	|| ( isset( $_REQUEST['page'] ) && 'eos_dp_code_browser' === $_REQUEST['page'] )
	|| ( isset( $_REQUEST['page'] ) && 'eos_dp_pro_assets' === $_REQUEST['page'] )
){
	add_action( 'admin_head',function(){
		$media = isset( $_REQUEST['export_file'] ) && 'true' === $_REQUEST['export_file'] ? 'all' : 'print';
		eos_dp_pro_inline_style_no_menu( $media );
	} );
	add_action( 'admin_init','eos_dp_pro_remove_admin_items',999999 );
	//Remove admin menu and top bar if saving as HTML file
	function eos_dp_pro_remove_admin_items(){
		add_action( 'admin_bar_menu','eos_dp_pro_remove_top_bar',999999);
	}

	add_action( 'admin_menu','eos_dp_pro_remmove_admin_menu',999999 );
	//Remove admin menu if saving as HTML file
	function eos_dp_pro_remmove_admin_menu(){
		global $menu,$submen;
		$menu = $submenu = array();
	}

	//Remove admin top bar if saving as HTML file
	function eos_dp_pro_remove_top_bar( $wp_admin_bar ){
		$all_toolbar_nodes = $wp_admin_bar->get_nodes();
		foreach ( $all_toolbar_nodes as $node ) {
			$wp_admin_bar->remove_node( $node->id );
		}
		return $wp_admin_bar;
	}
	remove_all_actions( 'in_dmin_footer' );
	remove_all_actions( 'admin_footer' );
	remove_all_actions( 'update_footer' );
	add_filter( 'admin_footer_text','__return_false',999999 );
	add_filter( 'update_footer','__return_false',999999 );
}
//Inline style to remove admin menu and admin top bar
function eos_dp_pro_inline_style_no_menu( $media = 'all' ){
	?><style id="eos-dp-pro-no-admin-menu" type="text/css" media="<?php echo esc_attr( $media ); ?>">
	#fdp-visit-site,#wpbody-content>div:not(.CodeMirror) {
    display: none
	}
	thead tr{
		position:sticky
	}
	#wpcontent {
			margin-left: 0 !important;
			margin-right: 0 !important;
			padding-left: 6px !important;
			padding-right: 6px !important;
	}
	#dolly,.button,#wpadminbar,#adminmenumain,#screen-meta-links,#wpfooter,#eos-dp-setts-nav-wrp,.eos-pre-nav,.eos-dp-report-row-actions,<?php echo 'eos_dp_pro_assets' !== $_REQUEST['page'] ? 'th:last-child,td:last-child,' : ''; ?>.eos-dp-order-rows{
			display: none !important;
	}
	.dashboard_page_eos_dp_code_browser .CodeMirror.CodeMirror-wrap {
    height: calc(100vh - 50px)
	}
	.dashboard_page_eos_dp_code_browser{
		overflow-y:hidden
	}
	html.wp-toolbar{
		padding-top:0
	}
	</style><?php
}


add_filter( 'all_plugins','eos_dp_pro_plugins_in_list_user' );
//Remove plugins from plugins table in the page wp-admin/plugins.php according to the Users FDP Settings
function eos_dp_pro_plugins_in_list_user( $plugins ){
	global $fdp_disabled_plugins_for_user;
	if( $fdp_disabled_plugins_for_user && is_array( $fdp_disabled_plugins_for_user ) ){
		foreach( $fdp_disabled_plugins_for_user as $plugin ){
			if( isset( $plugins[$plugin] ) ){
				unset( $plugins[$plugin] );
			}
		}
		return $plugins;
	}
	return $plugins;
}
add_filter( 'all_plugins','eos_dp_pro_plugins_in_list_setts' );
//Remove plugins from plugins table in the page wp-admin/plugins.php according to the Plugins FDP Settings
function eos_dp_pro_plugins_in_list_setts( $plugins ){
	$opts = eos_dp_pro_get_option( 'eos_dp_pro_main' );
	$opts = isset( $opts['plugins_setts'] ) ? $opts['plugins_setts'] : array();
	if( isset( $opts['v'] ) ){
		$hidden = $opts['v'];
		foreach( array_keys( $plugins ) as $plugin ){
			if( isset( $hidden[sanitize_key( dirname( $plugin ) )] ) && !$hidden[sanitize_key( dirname( $plugin ) )]  && EOS_DP_PLUGIN_BASE_NAME !== $plugin && EOS_DP_PRO_PLUGIN_BASE_NAME !== $plugin ){
				unset( $plugins[$plugin] );
			}
		}
		return $plugins;
	}
	return $plugins;
}

//Labels of GTMetrix
function eos_dp_pro_gtmetrix_labels(){
	return array(
		'page_url' => esc_html__( 'Page','eos-dp-pro' ),
		'disabled_plugins' => esc_html__( 'Disabled Plugins%s','eos-dp-pro' ),
		'location' => esc_html__( 'Location','eos-dp-pro' ),
		'onload_time' => esc_html__( 'Onload [ms]','eos-dp-pro' ),
		'page_load_time' => esc_html__( 'Page Load [ms]','eos-dp-pro' ),
		'fully_loaded_time' => esc_html__( 'Fully Loaded [ms]','eos-dp-pro' ),
		'html_load_time' => esc_html__( 'HTML Load [ms]','eos-dp-pro' ),
		'redirect_duration' => esc_html__( 'Redirect [ms]','eos-dp-pro' ),
		'dom_content_loaded_time' => esc_html__( 'DOM Loaded [ms]','eos-dp-pro' ),
		'dom_interactive_time' => esc_html__( 'DOM Interaction [ms]','eos-dp-pro' ),
		'page_bytes' => esc_html__( 'Page Size [kB]','eos-dp-pro' ),
		'onload_duration' => esc_html__( 'Onload Duration [ms]','eos-dp-pro' ),
		'connect_duration' => esc_html__( 'Connect [ms]','eos-dp-pro' ),
		'pagespeed_score' => esc_html__( 'Page Speed Score','eos-dp-pro' ),
		'yslow_score' => esc_html__( 'Yslow Score','eos-dp-pro' ),
		'report_url' => esc_html__( 'Report','eos-dp-pro' ),
	);
}

//Labels of Google PSI
function eos_dp_pro_gpsi_labels(){
	return array(
		'page_url' => esc_html__( 'Page','eos-dp-pro' ),
		'theme' => esc_html__( 'Theme','eos-dp-pro' ),
		'disabled_plugins' => esc_html__( 'Disabled Plugins%s','eos-dp-pro' ),
		'device' => esc_html__( 'Device','fdp-pro' ),
		'cls' => esc_html__( 'Cumulative layout shift [ms]','fdp-pro' ),
		'si' => esc_html__( 'Speed index [ms]','fdp-pro' ),
		'fcp' => esc_html__( 'First contentful paint [ms]','fdp-pro' ),
		'lcp' => esc_html__( 'Largest contentful paint [ms]','fdp-pro' ),
		'tbt' => esc_html__( 'Total blocking time [ms]','fdp-pro' ),
		'fmp' => esc_html__( 'First meaningful paint [ms]','fdp-pro' ),
		'srt' => esc_html__( 'Server response time [ms]','fdp-pro' ),
		't' => esc_html__( 'Total timing [ms]','fdp-pro' ),
		'score' => esc_html__( 'Score','fdp-pro' ),
	);
}

add_action( 'eos_dp_after_footer','eos_dp_pro_deactivation_page_after_foooter' );

function eos_dp_pro_deactivation_page_after_foooter(){
	if( function_exists( 'eos_dp_is_deactivation_page' ) && eos_dp_is_deactivation_page() ){
		wp_nonce_field( 'fdp_pro_deactivation_page','fdp_pro_deactivation_page' );
	}
}

if( defined( 'FDP_DEV_MODE' ) && false !== FDP_DEV_MODE ){
	add_action( 'admin_bar_menu','eos_dp_pro_visit_site_dev_mode', 60 );
	add_filter( 'post_row_actions','eos_dp_pro_dev_mode_action_links' );
	add_filter( 'page_row_actions','eos_dp_pro_dev_mode_action_links' );
	add_filter( 'get_sample_permalink_html', 'eos_dp_pro_top_form_edit_dev_mode',5,5 );
}
// Add an option to visit the site in FDP development mode.
function eos_dp_pro_visit_site_dev_mode( $wp_admin_bar ){
	$wp_admin_bar->add_node( array(
		'parent' => 'site-name',
		'id'     => 'fdp-dev-mode',
		'title'  => esc_attr__( 'Visit Site in FDP dev mode', 'eos-dp-pro' ),
		'href'   => esc_url( add_query_arg( 'fdp-dev',FDP_DEV_MODE,get_home_url() ) ),
	) );
}


//It adds the FDP dev mode query arg to the view post/page urls in the backend lists
function eos_dp_pro_dev_mode_action_links( $actions ) {
	if( !isset( $actions['view'] ) ) return $actions;
	$arr = explode( 'href="',$actions['view'] );
	$arr = explode( '"',$arr[1] );
	$actions['view_fdp_dev'] = '<a href="'.add_query_arg( 'fdp-dev',FDP_DEV_MODE,$arr[0] ).'" target="_blank">'.esc_html__( 'View in FDP dev mode','eos-dp-pro' ).'</a>';
	return $actions;
}

//It adds the FDP dev mode query arg to the view post/page urls in the editor screen
function eos_dp_pro_top_form_edit_dev_mode( $return, $id,$new_title,$new_slug,$post ){
	$url = add_query_arg( 'fdp-dev',FDP_DEV_MODE,get_permalink( $post->ID ) );
	$return .= '<a class="button button-small" href="'.$url.'" target="_blank">'.esc_html__( 'View page in FDP dev mode','eos-dp-pro' ).'</a>';
	return $return;
}

add_filter( 'pre_set_site_transient_update_plugins','eos_dp_pro_filter_plugin_updates' );
add_filter( 'pre_set_transient_update_plugins','eos_dp_pro_filter_plugin_updates' );
//Manage plugin update notifications
function eos_dp_pro_filter_plugin_updates( $value ) {
	$opts = eos_dp_pro_get_option( 'eos_dp_pro_main' );
	if( $opts && isset( $opts['plugins_setts'] ) ){
		$opts = $opts['plugins_setts'];
		if( $opts && isset( $opts['u'] ) ){
			$opts = $opts['u'];
			$plugins = array_unique( array_keys( get_plugins() ) );
			foreach( $plugins as $plugin ){
				if( in_array( sanitize_key( dirname( $plugin ) ),array_keys( $opts ) ) && !$opts[sanitize_key( dirname( $plugin ) )] && !in_array( $plugin,array( EOS_DP_PLUGIN_BASE_NAME,EOS_DP_PRO_PLUGIN_BASE_NAME ) ) ){
					unset( $value->response[$plugin] );
				}
			}
		}
	}
  return $value;
}

add_action( 'fdp_define_pointers',function(){
	//Load pointers
	require EOS_DP_PRO_PLUGIN_DIR.'/admin/pointers/pointers-pro.php';
} );

add_action( 'fdp_top_bar_notifications','eos_dp_pro_after_plugin_activation_notice' );
//Notice in the backend after a pllugin is activated.
function eos_dp_pro_after_plugin_activation_notice(){
	if( !defined( 'EOS_DP_VERSION' ) || !function_exists( 'eos_dp_active_plugins' ) ) return;
	$transient = get_site_transient( 'fdp_plugin_activated_notification' );
	if( $transient ){
		$active_plugins = eos_dp_active_plugins();
		$href = admin_url( 'admin.php?page=eos_dp_pro_bulk_actions' );
		if( $active_plugins && is_array( $active_plugins ) && in_array( $transient,$active_plugins ) ){
			eos_dp_display_admin_notice(
					'after_plugin_activation',
					__( 'New plugin activated.','eos-dp-pro' ),
					wp_kses(
					 sprintf(
						 __( 'The plugin %s was recently activated. You can activate/deactivate it in the FDP settings in the %sBulk Actions page%s.','eos-dp-pro' ),
						 strtoupper( str_replace( '-',' ',dirname( $transient ) ) ),
						 '<a href="'.$href.'" target="_blank">',
						 '</a>'
					 ),
					 array( 'a' => array( 'href' => array(),'target' => array() ) )
				 	),
					'warning'
			);
		}
	}
}

add_filter( 'eos_dp_integration_action_plugins',function( $args ){
    $args['fdp-pro-ajax-recording'] = array(
        'is_active' => defined( 'FDP_PRO_ACTIVE' ),
        'ajax_actions' => array(
            'eos_dp_pro_record_ajax_action_start' => array( 'description' => __( 'Ajax recording start','eos-dp-pro' ) ),
            'eos_dp_pro_record_ajax_action_stop' => array( 'description' => __( 'Ajax recording stop','eos-dp-pro' ) ), )
    );
    return $args;
} );

add_action( 'eos_dp_before_tabs',function(){
?>
<li class="eos-dp-has-children hover<?php echo in_array( $_GET['page'],array( 'eos_dp_pro_autoload' ) ) ? ' eos-active' : ''; ?>  eos-dp-setts-menu-item">
	<a href="<?php echo admin_url( 'admin.php?page=eos_dp_pro_autoload' ); ?>" title="<?php esc_attr_e( 'Database','eos-dp' ); ?>"><span class="dashicons dashicons-database"></span></a>
	<ul class="eos-dp-sub-menu">
		<li class="hover<?php echo $_GET['page'] === 'eos_dp_pro_autoload' ? ' eos-active' : ''; ?> eos-dp-setts-menu-item"><a href="<?php echo admin_url( 'admin.php?page=eos_dp_pro_autoload' ); ?>"><?php esc_html_e( 'Autoload','eos-dp-pro' ); ?></a></li>
	</ul>
</li>
<?php
} );

add_filter( 'fdp_pages',function( $fdp_pages ){
	$fdp_pages[] = 'eos_dp_backend_everywhere';
	$fdp_pages[] = 'eos_dp_by_post_requests';
	$fdp_pages[] = 'eos_dp_pro_whois';
	$fdp_pages[] = 'eos_dp_translation_urls';
	$fdp_pages[] = 'eos_dp_unlogged';
	$fdp_pages[] = 'eos_dp_cron';
	$fdp_pages[] = 'eos_dp_rest_api';
	$fdp_pages[] = 'eos_dp_by_plugin';
	$fdp_pages[] = 'eos_dp_conflicts';
	return array_unique( $fdp_pages );
} );

add_filter( 'fdp_main_nav_menu_items',function( $arr ){
	$arr['users'] = array(
		'title' =>  __( 'Users','eos-dp' ),
		'section' => 'logged',
		'active_if' => array( 'eos_dp_logged' ),
		'subitems' => array( 'eos_dp_logged','eos_dp_unlogged' ),
		'href' => admin_url( 'admin.php?page=eos_dp_logged' ),
		'file' => EOS_DP_PRO_PLUGIN_DIR.'/admin/templates/partials/nav-menu-items/menu-item-users.php',
	);
	return $arr;
} );

add_action( 'admin_menu',function(){
	if( function_exists( 'fdp_add_submenu_page' ) ){
		$menu_pages = array(
			array( 'miscellaneus','eos_dp_cron',__( 'Cron Jobs','eos-dp-pro' ),'activate_plugins','eos_dp_pro_cron_callback',10 ),
			array( 'miscellaneus','eos_dp_frontend_everywhere',__( 'Disabled on frontend','eos-dp-pro' ),'activate_plugins','eos_dp_pro_frontend_everywhere_callback',10 ),
			array( 'miscellaneus','eos_dp_frontend_untouchables',__( 'Active on frontend','eos-dp-pro' ),'activate_plugins','eos_dp_pro_frontend_untouchables_callback',20 ),
			array( 'miscellaneus','eos_dp_by_plugin',__( 'By plugin','eos-dp-pro' ),'activate_plugins','eos_dp_by_plugin_callback',30 ),
			array( 'miscellaneus','eos_dp_conflicts',__( 'Plugin Conflicts','eos-dp-pro' ),'activate_plugins','eos_dp_conflicts_callback',30 ),
			array( 'backend','eos_dp_backend_untouchables',__( 'Active on backend','eos-dp-pro' ),'activate_plugins','eos_dp_pro_backend_untouchables_callback',10 ),
			array( 'url','eos_dp_rest_api',__( 'Rest API','eos-dp-pro' ),'activate_plugins','eos_dp_pro_rest_api_callback',30 ),
		);
		if( version_compare( EOS_DP_VERSION,'2.1.7' ) > 0 ){
			$menu_pages[] = array( 'miscellaneus','eos_dp_browser',__( 'User Agents','eos-dp-pro' ),'activate_plugins','eos_dp_browser_callback',60 );
		}
		foreach( $menu_pages as $menu_page ) {
			if( isset( $_GET['page'] ) && $menu_page[1] === sanitize_text_field( $_GET['page'] ) ) {
				$callback = $menu_page[4];
				$file = EOS_DP_PRO_PLUGIN_DIR.'/admin/templates/pages/fdp-' . str_replace( array( 'eos_dp_pro_', 'eos_dp_', '_' ), array( '', '', '-' ), $menu_page[1] ) . '.php';
				if( file_exists( $file ) ) {
					require_once sanitize_text_field( $file );
				}
				else {
					$callback = '__return_false';
				}
				
			}
			else{
				$callback = '__return_false';
			}
			fdp_add_submenu_page( 
				$menu_page[0],
				$menu_page[1],
				$menu_page[2],
				$menu_page[3],
				$callback,
				$menu_page[5]
			);
		}

	}
} );

add_action( 'fdp_admin_notices',function(){
	$opts = eos_dp_pro_get_option( 'eos_dp_pro_main' );
	$opts = isset( $opts['eos_dp_general_setts'] ) ? $opts['eos_dp_general_setts'] : array();
	if( isset( $opts['dev_mode'] ) &&  '1' === $opts['dev_mode']	){
		$dev_url = esc_url( add_query_arg( 'fdp-dev',substr( md5( sanitize_text_field( $_SERVER['HTTP_HOST'] ) ),0,8 ),get_home_url() ) );
	?>
	<div class="fdp-wrp notice notice-alert is-dismissible" style="line-height:1.5;display:block !important">
		<p><?php esc_html_e( 'Development mode active.','eos-dp-pro' ); ?></p>
		<p><span><?php echo wp_kses( sprintf( __( 'If the Development mode is active, FDP will work on the frontend only by adding the query argument ?fdp-dev=%s to the URL, e.g. %s%s%s','eps-dp-pro' ),$dev,'<a href="'.$dev_url.'" target="_blank">',$dev_url,'</a>' ),array( 'a' => array( 'href' => array(),'target' => '_blank' ) ) ); ?></span></p>
		<?php if( !isset( $_GET['page'] ) || 'eos_dp_pro_settings' !== $_GET['page'] ){ ?>
		<p><?php printf( esc_html__( 'You can disable the devopment mode in the %sMain Settings%s.','eos-dp-pro' ),'<a title="'.esc_attr( __( 'Link to the FDP main settings page','eos-dp-pro' ) ).'" href="'.add_query_arg( 'page','eos_dp_pro_settings',admin_url() ).'" target="_fdp_main_settings">','</a>' ); ?></p>
		<?php } ?>
	</div>
	<?php
	}
} );

if( defined( 'FDP_PRO_LICENSE_EDD' ) && FDP_PRO_LICENSE_EDD ){
	define( 'EOS_DP_PRO_EDD_ITEM_ID',2672 );
	define( 'EOS_DP_PRO_EDD_ITEM_NAME','Freesoul Deactivate Plugins PRO' );
	define( 'EOS_DP_PRO_EDD_STORE_URL','https://shop.freesoul-deactivate-plugins.com/edd/?fdp-edd-lic=true' );
	/**
	 * Adds content to the settings section.
	 *
	 * @return void
	 */
	function eos_dp_pro_edd_license_key_settings_section() {
		// esc_html_e( 'This is where you enter your license key.' );
	}

	/**
	 * Outputs the license key settings field.
	 *
	 * @return void
	 */
	function eos_dp_pro_edd_license_key_settings_field() {
		$license = get_option( 'eos_dp_pro_edd_license_key' );
		$status  = get_option( 'eos_dp_pro_edd_license_status' );
		// wp_enqueue_script( 'fdp-settings',EOS_DP_SETTINGS_JS_URL,array(),true );
		// wp_localize_script( 'fdp-settings','fdp_setts_js',array( 'ajaxurl' => admin_url( 'admin-ajax.php' ),'action' => 'eos_dp_pro_save_settings','opts_key' => 'eos_dp_pro_edd_license_key' ) );
		?>
		<p class="description"><?php esc_html_e( 'Enter your license key.' ); ?></p>
		<?php
		printf(
			'<input type="text" class="regular-text" id="eos_dp_pro_edd_license_key" name="eos_dp_pro_edd_license_key" style="padding:5px;width:20em;margin:0" value="%s" />',
			esc_attr( $license )
		);
		$button = array(
			'name'  => 'edd_license_deactivate',
			'label' => __( 'Deactivate License' ),
		);
		if ( 'valid' !== $status ) {
			$button = array(
				'name'  => 'edd_license_activate',
				'label' => __( 'Activate License' ),
			);
		}
		wp_nonce_field( 'fdp_check_license_status', 'fdp_check_license_status' );
		?>
		<input type="submit" class="button" name="<?php echo esc_attr( $button['name'] ); ?>" value="<?php echo esc_attr( $button['label'] ); ?>"/>
		<?php
	}

	/**
	 * Registers the license key setting in the options table.
	 *
	 * @return void
	 */
	function eos_dp_pro_edd_register_option() {
		register_setting( 'eos_dp_pro_edd_license', 'eos_dp_pro_edd_license_key', 'edd_sanitize_license' );
	}
	add_action( 'admin_init', 'eos_dp_pro_edd_register_option' );

	/**
	 * Sanitizes the license key.
	 *
	 * @param string  $new The license key.
	 * @return string
	 */
	function edd_sanitize_license( $new ) {
		$old = get_option( 'eos_dp_pro_edd_license_key' );
		if ( $old && $old !== $new ) {
			delete_option( 'eos_dp_pro_edd_license_status' ); // new license has been entered, so must reactivate
		}

		return sanitize_text_field( $new );
	}

	/**
	 * Activates the license key.
	 *
	 * @return void
	 */
	function eos_dp_pro_edd_activate_license() {

		// // listen for our activate button to be clicked
		// if ( ! isset( $_POST['edd_license_activate'] ) ) {
		// 	return;
		// }

		// // run a quick security check
		// if ( ! check_admin_referer( 'fdp_check_license_status', 'fdp_check_license_status' ) ) {
		// 	return; // get out if we didn't click the Activate button
		// }

		// // retrieve the license from the database
		// $license = trim( get_option( 'eos_dp_pro_edd_license_key' ) );
		// if ( ! $license ) {
		// 	$license = ! empty( $_POST['eos_dp_pro_edd_license_key'] ) ? sanitize_text_field( $_POST['eos_dp_pro_edd_license_key'] ) : '';
		// }
		// if ( ! $license ) {
		// 	return;
		// }

		// // data to send in our API request
		// $api_params = array(
		// 	'edd_action'  => 'activate_license',
		// 	'license'     => $license,
		// 	'item_id'     => EOS_DP_PRO_EDD_ITEM_ID,
		// 	'item_name'   => rawurlencode( EOS_DP_PRO_EDD_ITEM_NAME ), // the name of our product in EDD
		// 	'url'         => home_url(),
		// 	'environment' => function_exists( 'wp_get_environment_type' ) ? wp_get_environment_type() : 'production',
		// );
		// 	// make sure the response came back okay

		// 	//$license_data = json_decode( wp_remote_retrieve_body( $response ) );
		// 	$license_data = new stdClass();
		// 	$license_data->success = true;
		// 	$license_data->license = 'valid';
			

		// 	if ( false === $license_data->success ) {

		// 		switch ( $license_data->error ) {

		// 			case 'expired':
		// 				$message = sprintf(
		// 					/* translators: the license key expiration date */
		// 					__( 'Your license key expired on %s.', 'edd-sample-plugin' ),
		// 					date_i18n( get_option( 'date_format' ), strtotime( $license_data->expires, current_time( 'timestamp' ) ) )
		// 				);
		// 				break;

		// 			case 'disabled':
		// 			case 'revoked':
		// 				$message = __( 'Your license key has been disabled.', 'edd-sample-plugin' );
		// 				break;

		// 			case 'missing':
		// 				$message = __( 'Invalid license.', 'edd-sample-plugin' );
		// 				break;

		// 			case 'invalid':
		// 			case 'site_inactive':
		// 				$message = __( 'Your license is not active for this URL.', 'edd-sample-plugin' );
		// 				break;

		// 			case 'item_name_mismatch':
		// 				/* translators: the plugin name */
		// 				$message = sprintf( __( 'This appears to be an invalid license key for %s.', 'edd-sample-plugin' ), EOS_DP_PRO_EDD_ITEM_NAME );
		// 				break;

		// 			case 'no_activations_left':
		// 				$message = __( 'Your license key has reached its activation limit.', 'edd-sample-plugin' );
		// 				break;

		// 			default:
		// 				$message = __( 'An error occurred, please try again.', 'edd-sample-plugin' );
		// 				break;
		// 		}
		// 	}
		// }

		// 	// Check if anything passed on a message constituting a failure
		// if ( ! empty( $message ) ) {
		// 	$redirect = add_query_arg(
		// 		array(
		// 			'page'          => 'eos_dp_pro_license',
		// 			'sl_activation' => 'false',
		// 			'message'       => rawurlencode( $message ),
		// 		),
		// 		admin_url( 'plugins.php' )
		// 	);

		// 	wp_safe_redirect( $redirect );
		// 	exit();
		// }

		// $license_data->license will be either "valid" or "invalid"
		//if ( 'valid' === $license_data->license ) {
			update_option( 'eos_dp_pro_edd_license_key', $license );
		//}
		update_option( 'eos_dp_pro_edd_license_status', $license_data->license );
		wp_safe_redirect( admin_url( 'plugins.php?page=' . 'eos_dp_pro_license' ) );
		exit();
	}
	add_action( 'admin_init', 'eos_dp_pro_edd_activate_license' );

	/**
	 * Deactivates the license key.
	 * This will decrease the site count.
	 *
	 * @return void
	 */
	function eos_dp_pro_edd_deactivate_license() {

		// listen for our activate button to be clicked
		if ( isset( $_POST['edd_license_deactivate'] ) ) {

			// run a quick security check
			if ( ! check_admin_referer( 'fdp_check_license_status', 'fdp_check_license_status' ) ) {
				return; // get out if we didn't click the Activate button
			}

			// retrieve the license from the database
			$license = trim( get_option( 'eos_dp_pro_edd_license_key' ) );

			// data to send in our API request
			$api_params = array(
				'edd_action'  => 'deactivate_license',
				'license'     => $license,
				'item_id'     => EOS_DP_PRO_EDD_ITEM_ID,
				'item_name'   => rawurlencode( EOS_DP_PRO_EDD_ITEM_NAME ), // the name of our product in EDD
				'url'         => home_url(),
				'environment' => function_exists( 'wp_get_environment_type' ) ? wp_get_environment_type() : 'production',
			);

			// decode the license data
			$license_data = json_decode( wp_remote_retrieve_body( $response ) );
			$license_data = new stdClass();
			$license_data->success = true;
			$license_data->license = 'deactivated';

			// $license_data->license will be either "deactivated" or "failed"
			if ( 'deactivated' === $license_data->license ) {
				delete_option( 'eos_dp_pro_edd_license_status' );
			}

			wp_safe_redirect( admin_url( 'plugins.php?page=' . 'eos_dp_pro_license' ) );
			exit();

		}
	}
	add_action( 'admin_init', 'eos_dp_pro_edd_deactivate_license' );

	/**
	 * Checks if a license key is still valid.
	 * The updater does this for you, so this is only needed if you want
	 * to do somemthing custom.
	 *
	 * @return void
	 */
	function eos_dp_pro_edd_check_license() {

		$license = trim( get_option( 'eos_dp_pro_edd_license_key' ) );

		$api_params = array(
			'edd_action'  => 'check_license',
			'license'     => $license,
			'item_id'     => EOS_DP_PRO_EDD_ITEM_ID,
			'item_name'   => rawurlencode( EOS_DP_PRO_EDD_ITEM_NAME ),
			'url'         => home_url(),
			'environment' => function_exists( 'wp_get_environment_type' ) ? wp_get_environment_type() : 'production',
		);

			$license_data = new stdClass();
			$license_data->success = true;
			$license_data->license = 'valid';

		if ( 'valid' === $license_data->license ) {
			echo 'valid';
			exit;
			// this license is still valid
		} else {
			echo 'invalid';
			exit;
			// this license is no longer valid
		}
	}

	/**
	 * This is a means of catching errors from the activation method above and displaying it to the customer
	 */
	function eos_dp_pro_edd_admin_notices() {
		if ( isset( $_GET['sl_activation'] ) && ! empty( $_GET['message'] ) ) {

			switch ( $_GET['sl_activation'] ) {

				case 'false':
					$message = urldecode( $_GET['message'] );
					?>
					<div class="error">
						<p><?php echo wp_kses_post( $message ); ?></p>
					</div>
					<?php
					break;

				case 'true':
				default:
					// Developers can put a custom success message here for when activation is successful if they way.
					break;

			}
		}
	}
	add_action( 'admin_notices', 'eos_dp_pro_edd_admin_notices' );
}