<?php
/**
 *  It handles the admin ajax requests
 */
defined( 'ABSPATH' ) || exit; // Exit if accessed directly.

do_action( 'fdp_pro_before_ajax_request' );

add_action( 'wp_ajax_eos_dp_pro_paste_from_post_types','eos_dp_pro_paste_from_post_types' );

//Send the post type options to copy them in a row
function eos_dp_pro_paste_from_post_types(){
	if( !isset( $_POST['post_type'] ) ){
		die();
		exit;
	}
	eos_dp_pro_check_intentions_and_rights( 'fdp_pro_deactivation_page' );
	$opts = eos_dp_plugins_table();
	if( isset( $opts[$_POST['post_type'] ] ) ){
		$opts = $opts[sanitize_text_field( $_POST['post_type'] )];
		if( isset( $opts[1] ) ){
			echo sanitize_text_field( $opts[1] );
			die();
			exit;
		}
	}
	die();
	exit;
}
add_action( 'wp_ajax_eos_dp_pro_record_ajax_action_start', 'eos_dp_pro_record_ajax_action_start' );
//Start the record to get the ajax action name
function eos_dp_pro_record_ajax_action_start(){
	if( isset( $_POST['action_type'] ) ){
		eos_dp_pro_check_intentions_and_rights( 'eos_dp_pro_record_'.sanitize_key( $_POST['action_type' ] ).'_action_start' );
		$session_id = isset( $_COOKIE['PHPSESSID'] ) ? $_COOKIE['PHPSESSID'] : '';
		set_site_transient( 'eos_dp_recording_session_id_'.sanitize_text_field( $session_id ),sanitize_text_field( $_POST['rec_id'] ),60*10 );
		set_site_transient( 'eos_dp_recording_action_'.sanitize_text_field( $_POST['rec_id'] ),'start',60*10 );
		if( 'post' === $_POST['action_type'] ){
			set_site_transient( 'eos_dp_recording_post_'.sanitize_text_field( $_POST['rec_id'] ),'true',60 );
		}
	}
	echo 1;
	die();
	exit;
}
add_action( 'wp_ajax_eos_dp_pro_record_ajax_action_stop', 'eos_dp_pro_record_ajax_action_stop',0 );
//Stop the recording to get the ajax action name
function eos_dp_pro_record_ajax_action_stop(){
	if( !isset( $_POST['rec_id'] ) && !isset( $_POST['action_type' ] ) ) {
	   echo 0;
	   die();
	   exit;
	}
	eos_dp_pro_check_intentions_and_rights( 'eos_dp_pro_record_'.sanitize_key( $_POST['action_type' ] ).'_action_stop' );
	$rec_id = sanitize_text_field( $_POST['rec_id'] );
  	$session_id = isset( $_COOKIE['PHPSESSID'] ) ? $_COOKIE['PHPSESSID'] : '';
	$transient = get_site_transient( 'eos_dp_recording_action_'.$rec_id );
	delete_site_transient( 'eos_dp_recording_action_'.$rec_id );
	delete_site_transient( 'eos_dp_recording_session_id_'.sanitize_text_field( $session_id ) );
	delete_site_transient( 'eos_dp_recording_post_'.$rec_id );
	if( $transient && is_array( $transient ) ){
	  $actions = wp_json_encode( array_unique( $transient ) );
		echo sanitize_text_field( $actions );
	}
	die();
	exit;
}

add_action( 'wp_ajax_eos_dp_pro_save_ajax_settings', 'eos_dp_pro_save_ajax_settings' );
//Saves admin options
function eos_dp_pro_save_ajax_settings(){
	if( isset( $_POST['data'] ) && isset( $_POST['action_names'] ) ){
		$action_type = isset( $_POST['action_type'] ) ? sanitize_text_field( $_POST['action_type'] ) : 'ajax';
		eos_dp_pro_check_intentions_and_rights( 'eos_dp_pro_get_'.sanitize_key( $action_type ).'_actions' );
		eos_dp_pro_update_option( 'eos_dp_new_plugin_activated',false );
		$all_actions = !empty( $_POST['data'] ) ? json_decode( str_replace( '\\','',$_POST['data'] ),true ) : array();
		$action_names = !empty( $_POST['action_names'] ) ? json_decode( str_replace( '\\','',$_POST['action_names'] ),true ) : array();
    $ajax_options = array();
    if( !empty( $all_actions ) ){
			foreach( $all_actions as $key => $value ){
        if( $key && !empty( $key ) ){
					$ajax_options[sanitize_text_field( str_replace( ',','--',$key ) )] = sanitize_text_field( $value );
        }
			}
		}
		$action_key = 'ajax' === $action_type ? 'action' : 'post_action';
		eos_dp_pro_update_option( 'eos_dp_pro_'.sanitize_key( $action_type ).'_setts',sanitize_text_field( wp_json_encode( $ajax_options ) ) );
		eos_dp_pro_update_option( 'eos_dp_pro_'.$action_key.'_names',sanitize_text_field( wp_json_encode( $action_names ) ) );
		if( isset( $_POST['theme_activation'] ) ){
			eos_dp_pro_update_option( 'eos_dp_pro_'.sanitize_key( $action_type ).'_theme',json_decode( str_replace( '\\','',sanitize_text_field( $_POST['theme_activation'] ) ),true ) );
		}
		echo 1;
	}
	die();
	exit;
}

add_action( 'wp_ajax_eos_dp_pro_errors_check', 'eos_dp_pro_errors_check' );
//Errors check
function eos_dp_pro_errors_check(){
	$nonce = isset( $_POST['nonce'] ) ? $_POST['nonce'] : false;
	if( !isset( $_POST['post_id'] ) ){
	   echo 0;
	   die();
	   exit;
	}
	eos_dp_pro_check_intentions_and_rights( 'eos_dp_pro_errors_check' );
	$time0 = time();
	set_site_transient( 'eos_dp_pro_scanning_error_plugins','true' );
	$post_id = absint( $_POST['post_id'] );
	$plugins = eos_dp_active_plugins();
	if( $plugins ){
		$giving_errors = array();
		$post_check = array();
		$permalink = get_permalink( $post_id );
		set_transient( 'fdp_test_'.$post_id,'',30 );
		$url = urldecode( add_query_arg( array( 'eos_dp_debug' => 'display_errors','eos_dp_preview' => 'true','eos_dp_pro_id' => md5( EOS_DP_PRO_TESTING_UNIQUE_ID ),'get_errors' => 'true','test_id' => time(),'fdp_post_id' => $post_id ),$permalink ) );
		$n = 0;
		foreach( $plugins as $plugin ){
			$activePlugins = $plugins;
			$k = array_search( $plugin,$activePlugins );
			if( false !== $k ){
				unset( $activePlugins[$k] );
			}
			$errArr = array();
			$paths_var = ';pn:'.implode( ';pn:',$activePlugins );
			//if pn is not empty that plugin is disabled
			set_transient( 'fdp_test_'.$post_id,$paths_var,20 );
			$response = wp_remote_get( $url,array( 'sslverify' => false ) );
			if( is_wp_error( $response ) ) continue;
			$body = wp_remote_retrieve_body( $response );

			$err = false;
			$errArr = explode( 'eos-dp-errors-start',$body );
			$errArr = isset( $errArr[1] ) ? explode( 'eos-dp-errors-end',$errArr[1] ) : false;
			$errObj = json_decode( $errArr[0] );
			$types2track = array(
				'1', //Fatal run-time errors. These indicate errors that can not be recovered from, such as a memory allocation problem. Execution of the script is halted.
				'4', //Compile-time parse errors. Parse errors should only be generated by the parser.
				'2', //Run-time warnings (non-fatal errors). Execution of the script is not halted.
				'8' //	Run-time notices. Indicate that the script encountered something that could indicate an error, but could also happen in the normal course of running a script.
			);
			if(
				false === strpos( $body,'eos-dp-pro-no-fatal-errors-occured' )
				|| (
					is_object( $errObj )
					&& '' !== $errObj->message
					&& in_array( $errObj->type,$types2track )
				)
			){
				$giving_errors[] = $plugin;
			}
			++$n;
		}
		if( !empty( $giving_errors ) ){
			echo json_encode( $giving_errors );
			die();
			exit;
		}
	}
	delete_transient( 'fdp_test_'.$post_id );
	delete_site_transient( 'eos_dp_pro_scanning_error_plugins' );
	die();
}

add_action( 'wp_ajax_eos_dp_pro_save_plugins_settings','eos_dp_pro_save_plugins_settings' );
//Plugin settings saving
function eos_dp_pro_save_plugins_settings(){
	$data = json_decode( sanitize_text_field( stripslashes(  $_POST['data'] ) ),true );
	if( !isset( $data['nonce'] ) ){
	   echo 0;
	   die();
	   exit;
	}
	if (
		! wp_verify_nonce( $data['nonce'],'fdp_plugins_setts_nonce' )
		|| !current_user_can( 'activate_plugins' )
	) {
		 echo 0;
		 die();
		 exit;
	}
	$plugins = array_unique( array_keys( get_plugins() ) );
	$opts = get_site_option( 'eos_dp_pro_main' );
	$opts_p = isset( $opts['plugins_setts'] ) ? $opts['plugins_setts'] : array();
	$opts['last_save'] = time();
	foreach( array( 'as','u','v' ) as $kp ){
		if( !isset( $opts_p[$kp] ) ){
			$opts_p[$kp] = array();
		}
		foreach( $plugins as $p ){
			if( isset( $data[$kp.'_'.sanitize_key( dirname( $p ) )] ) ){
				$opts_p[$kp][sanitize_key( dirname( $p ) )] = sanitize_text_field( $data[$kp.'_'.sanitize_key( dirname( $p ) )] );
			}
		}
	}
	delete_transient( 'update_plugins' );
	delete_site_transient( 'update_plugins' );
	$opts['plugins_setts'] = $opts_p;
	if( update_site_option( 'eos_dp_pro_main',$opts,true  ) ){
		echo 1;
		die();
		exit;
	}
	echo 0;
	die();
	exit;
}
add_action( 'wp_ajax_eos_dp_pro_save_general_bloat_settings','eos_dp_pro_save_general_bloat_settings' );
//General bloat saving
function eos_dp_pro_save_general_bloat_settings(){
	if( !isset( $_POST['nonce'] ) ){
	   echo 0;
	   die();
	   exit;
	}
	eos_dp_pro_check_intentions_and_rights( 'fdp_general_bloat_nonce' );
	$opts = get_site_option( 'eos_dp_pro_main' );
	if( !$opts || !is_array( $opts ) ){
		$opts = array();
	}
	if( !isset( $opts['general_bloat'] ) ){
		$opts['general_bloat'] = array();
	}
	$general_bloat = $opts['general_bloat'];
	$unneeded_actions = array_keys( eos_dp_pro_unneeded_actions() );
	foreach( $unneeded_actions as $key ){
		if( isset( $_POST[$key] ) ) $general_bloat[$key] = sanitize_text_field( $_POST[$key] );
	}
	foreach( array(
		'comment_spam',
		'global_style',
		'sworg_dns_prefetch',
		'xmlrpc',
		'emoji',
		'rest_api',
		'heartbeat',
		'dirsize_cache',
		'heartbeat_value',
		'revisions',
		'revisions_value',
		'cookies_assets',
		'jmigrate',
		'jquery',
		'cart_fragments',
		'woo_emails',
		'woo_blocks_style',
		'gfonts',
		'favicon',
		'gutenberg',
		'cron',
		'autosave'
	) as $key ){
		if( isset( $_POST[$key] ) ) $general_bloat[$key] = sanitize_text_field( $_POST[$key] );
	}
	$opts['general_bloat'] = $general_bloat;
	$opts['last_save'] = time();
	if( update_site_option( 'eos_dp_pro_main',$opts,true  ) ){
		echo 1;
		die();
		exit;
	}
	echo 0;
	die();
	exit;
}

add_action( 'wp_ajax_eos_dp_pro_save_by_plugin_settings', 'eos_dp_pro_save_by_plugin_settings' );
// Save By Plugin settings
function eos_dp_pro_save_by_plugin_settings() {
	if( ! isset( $_POST['page_slug'] ) ) {
		echo 0;
		die();
		exit;
	}
	$data = json_decode( sanitize_text_field( stripslashes(  $_POST['data'] ) ),true );
	if( !isset( $data['nonce'] ) ){
	   echo 0;
	   die();
	   exit;
	}
	if (
		! wp_verify_nonce( $data['nonce'],'eos_dp_by_plugin_nonce' )
		|| !current_user_can( 'activate_plugins' )
	) {
		 echo 0;
		 die();
		 exit;
	}
	$opts = array();
	foreach( $data as $k => $v ) {
		if( false !== strpos( $k, '.php' ) ) {
			$opts[ sanitize_text_field( $k ) ] = sanitize_text_field( $v );
		}
	}
	echo eos_dp_update_option( sanitize_text_field( $_POST['page_slug'] ), $opts );

	die();
	exit;
}

add_action( 'wp_ajax_eos_dp_pro_save_testing_settings', 'eos_dp_pro_save_testing_settings' );
//GT Metrix test
function eos_dp_pro_save_testing_settings(){
	if( !isset( $_POST['eos_dp_gtmetrix_email'] ) && !isset( $_POST['eos_dp_gtmetrix_api'] ) && !isset( $_POST['eos_dp_gpsi_api'] ) ){
	   echo 0;
	   die();
	   exit;
	}
	eos_dp_pro_check_intentions_and_rights( 'eos_dp_testing' );
	$opts = get_site_option( 'fdp_pro_admin' );
	$opts['gtmetrix_email'] = sanitize_email( $_POST['eos_dp_gtmetrix_email'] );
	$opts['gtmetrix_api'] = sanitize_text_field( $_POST['eos_dp_gtmetrix_api'] );
	$opts['gpsi_api'] = sanitize_text_field( $_POST['eos_dp_gpsi_api'] );
	$opts['gtmetrix_location_id'] = sanitize_text_field( $_POST['eos_dp_gtmetrix_location_id'] );
	$opts['last_save'] = time();
	if( update_site_option( 'fdp_pro_admin',$opts,true  ) ){
		echo 1;
		die();
		exit;
	}
	echo 0;
	die();
	exit;
}

add_action( 'wp_ajax_eos_dp_pro_autosuggest_col', 'eos_dp_pro_autosuggest_col' );
//Suggest where the plugin is useful
function eos_dp_pro_autosuggest_col(){
	if( ( !isset( $_POST['post_id'] ) && !isset( $_POST['admin_page'] ) ) || !isset( $_POST['plugin'] ) ){
	   echo 0;
	   die();
	   exit;
	}
	$args = array(
		'internal_call' => true,
		'nonce' => $_POST['nonce'],
	);
	if( isset( $_POST['post_id'] ) ){
		$args['post_id'] = absint( $_POST['post_id'] );
		$unused_plugins = eos_dp_pro_auto_settings(
			$args,
			array( sanitize_text_field( $_POST['plugin'] ) )
		);
	}
	elseif( isset( $_POST['admin_page'] ) ){
		$args['admin_page'] = esc_attr( $_POST['admin_page'] );
		$unused_plugins = eos_dp_pro_auto_settings_admin(
			$args,
			array( sanitize_text_field( $_POST['plugin'] ) )
		);
	}
	echo in_array( $_POST['plugin'],$unused_plugins );
	die();
	exit;
}

add_action( 'wp_ajax_eos_dp_pro_check_whois', 'eos_dp_pro_check_whois' );
//Find who is plugin
function eos_dp_pro_check_whois(){
	if( !current_user_can( 'activate_plugins' ) || !isset( $_POST['data'] ) ) return;
	$data = json_decode( stripslashes( $_POST['data'] ) );
	if( !isset( $data->nonce ) || !wp_verify_nonce( esc_attr( $data->nonce ),'eos_dp_whois_nonce' ) ){
		die();
		exit;
	}
	$microtime = 10000*microtime(1);
	$url_args = array(
		'display_usage' => 'false',
		'eos_dp_debug' => 'no_errors',
		'eos_dp_preview' => 'true',
		'eos_dp_pro_id' => md5( EOS_DP_PRO_TESTING_UNIQUE_ID ),
		'test_id' => $microtime,
		'tool' => 'whois',
		'site_in_progress' => substr( md5( esc_attr( get_option( 'comingsoon_input_psw_email' ) ).'_' ),0,8 )
	);
	$strcount = 0;
	$search = str_replace( '"','',str_replace( '"','',$data->search ) );
	$args = eos_dp_pro_get_user_headers( array( 'timeout' => 15,'sslverify' => false ),true );
	$count = $data->count;
	$plugins_count = $data->plugins_count;
	$plugin = $data->plugin;
	$url = esc_url( $data->url );
	$cookies = array();
	foreach ( $_COOKIE as $name => $value ) {
		$cookies[sanitize_key( $name )] = sanitize_text_field( $value );
	}
	$args['cookies'] = $cookies;
	$disabled_on_whois = array( 'query-monitor/query-monitor.php','template-cache/template-cache.php' );
	if( 'db' === $plugin ){
		$query = new WP_Query( array( 's' => $data->search ) );
		if( $query && is_object( $query ) && isset( $query->posts ) ){
			$posts = $query->posts;
			if( isset( $posts[0]->ID ) && absint( $posts[0]->ID ) > 0 ){
				echo wp_json_encode( array( 'found_in_posts' => array( 'url' => get_the_permalink( $posts[0]->ID ),'title' => esc_html( $posts[0]->post_title ) ) ) );
			}
			else{
				echo 'not_found';
			}
			die();
			exit;
		}
	}
	elseif( 'all' === $plugin ){
		set_transient( 'fdp_test_whois_'.$microtime,implode( ';pn:',array_filter( array_unique( $disabled_on_whois ) ) ),600 );
		$response_all = wp_remote_request( add_query_arg( $url_args,$url ),$args );
		if ( !is_wp_error( $response_all ) ){
			$html_all = wp_remote_retrieve_body( $response_all );
			if( $html_all ){
				$html_all = str_replace( "'",'',str_replace( '"','',$html_all ) );
				$strcount = substr_count( $html_all,str_replace( '"','',$search ) );
				set_site_transient( 'fdp_whois_'.sanitize_key( $data->url ).'_substrcount_all',$strcount,60 );
				if( false === strpos( str_replace( '"','',$html_all ),str_replace( '"','',$search ) ) ){
					delete_site_transient( 'fdp_whois_'.sanitize_key( $data->url ).'_substrcount_all' );
					echo 'notfound';
					die();
					exit;
				}
			}
			else{
				echo 'error';
				die();
				exit;
			}
		}
	}
	else{
		if( 'fdp_naked_theme' === $plugin ){
			$url_args['theme'] = 'fdp_naked';
			$plugin = '';
			$theme = wp_get_theme();
			if( 'fdp-theme' !== $theme->template ){
				$plugin_data = $theme && is_object( $theme ) ? array( 'Title' => $theme->get( 'Name'),'Author' => $theme->get( 'Author'),'Description' => $theme->get( 'Description' ) ) : array( 'Title' => 'Theme','Author' => '','Description' => '' );
			}
		}
		else{
			$plugin_data = get_plugin_data( WP_PLUGIN_DIR.'/'.$plugin );
		}
		$url = add_query_arg( $url_args,$url );
		set_transient( 'fdp_test_whois_'.$microtime,implode( ';pn:',array_filter( array_unique( array_merge( array( $plugin ),$disabled_on_whois ) ) ) ),1200 );
		$response = wp_remote_get( $url,$args );
		if ( !is_wp_error( $response ) ){
			$html = wp_remote_retrieve_body( $response );
			if( $html ){
				$html = str_replace( "'",'',str_replace( '"','',$html ) );
				$strcount = get_site_transient( 'fdp_whois_'.sanitize_key( $data->url ).'_substrcount_all' );
				if( false === strpos( $html,$search ) || substr_count( $html,$search ) < $strcount ){
					echo wp_json_encode( $plugin_data );
					die();
					exit;
				}
			}
			else{
				echo 'missed-'.$plugin;
			}
		}
		if( absint( $data->count ) === absint( $data->plugins_count ) - 1 ){
			delete_site_transient( 'fdp_whois_'.sanitize_key( $data->url ).'_substrcount_all' );
		}
	}
	echo 1;
	die();
	exit;
}

add_action( 'wp_ajax_eos_dp_pro_preview_page_from_report', 'eos_dp_pro_preview_page_from_report' );
//Preview page from report
function eos_dp_pro_preview_page_from_report(){
	if( !isset( $_POST['time'] ) || !isset( $_POST['tool'] ) ){
		die();
		exit;
	}
	eos_dp_pro_check_intentions_and_rights( 'eos_dp_pro_report' );
	$tool = isset( $_POST['tool'] ) ? sanitize_key( $_POST['tool'] ) : 'gtmetrix';
	$tool_key = 'last_'.$tool;
	$upload_dir = wp_upload_dir();
	$opts = eos_dp_pro_get_option( 'fdp_pro_admin' );
	$last_tool = isset( $opts[$tool_key] ) ? sanitize_title( $opts[$tool_key] ) : md5( time() );
	if( file_exists( $upload_dir['basedir'].'/FDP/reports/'.$tool.'/fdp-report-'.$last_tool.'.json' ) ){
		$json = file_get_contents( $upload_dir['basedir'].'/FDP/reports/'.$tool.'/fdp-report-'.$last_tool.'.json' );
		if( $json ){
			$arr = json_decode( $json,true );
			$time_key = $_POST['time'];
			if( isset( $arr[$time_key] ) ){
				$arr = $arr[$time_key];
				if( isset( $arr['disabled_plugins'] ) ){
					if( false !== strpos( $arr['disabled_plugins'],',' ) ){
						$active_plugins = eos_dp_active_plugins();
						$disabled_plugins_names = explode( ',',$arr['disabled_plugins'] );
						$active_plugins_names = array_map( 'eos_dp_name_by_path',$active_plugins );
						$disabled_keys = array_keys( array_intersect( $active_plugins_names,$disabled_plugins_names ) );
						$disabled_plugins = array();
						foreach( $disabled_keys as $key ){
							$disabled_plugins[] = $active_plugins[$key];
						}
						$eos_dp_paths = sanitize_text_field( implode( ';pn:',$disabled_plugins ) );
						set_transient( 'fdp_test_'.sanitize_key( $tool ).'_'.sanitize_key( $time_key ),sanitize_text_field( $eos_dp_paths ),60 );
						echo 1;
						die();
						exit;
					}
				}
			}
		}
	}
}


add_action( 'wp_ajax_eos_dp_pro_gt_metrix_test', 'eos_dp_pro_gt_metrix_test' );
//GT Metrix test
function eos_dp_pro_gt_metrix_test(){
	eos_dp_pro_check_intentions_and_rights( 'eos_dp_pro_gt_metrix_test' );
	if( isset( $_POST['post_id'] ) || ( isset( $_POST['post_type'] ) && isset( $_POST['archive_url'] ) ) ){
		$test_id = time();
		$theme = isset( $_POST['theme'] ) && 'false' !== $_POST['theme'] ? esc_attr( $_POST['theme'] ) : false;
		$dpOpts = get_site_option( 'fdp_pro_admin' );
		$user_name = sanitize_email( $dpOpts['gtmetrix_email'] );
		$api_key = esc_attr( $dpOpts['gtmetrix_api'] );
		if( isset( $_POST['post_id'] ) ){
			$post_id = absint( $_POST['post_id'] );
			$post = get_post( $post_id );
			$post_title = is_object( $post ) ? $post->post_title : '';
			$page_url = get_permalink( absint( $post_id ) );
		}
		else{
			$post_id = esc_attr( $_POST['post_type'] );
			$labels = get_post_type_labels( get_post_type_object( $post_id ) );
			$post_type_label = isset( $labels->name ) ? $labels->name : $post_id;
			$post_title = sanitize_text_field( sprintf( __( '%s Archive','eos-dp-pro' ),$post_type_label ) );
			$page_url = esc_url( $_POST['archive_url'] );
		}
		$url_to_test = add_query_arg( array( 'eos_dp_preview' => 'true','tool' => 'gtmetrix','fdp_post_id' => $post_id,'test_id' => $test_id  ),$page_url );
		if( $theme ){
				$url_to_test = add_query_arg( 'theme',$theme,$url_to_test );
		}
		if( isset( $_POST['plugins_path'] ) ){
			set_transient( 'fdp_test_gtmetrix_'.$post_id,sanitize_text_field( $_POST['plugins_path'] ),300 );
		}
		require_once( EOS_DP_PRO_PLUGIN_DIR.'/inc/gtmetrix-testing.php' );
		set_transient( 'fdp_testing_nonce_'.sanitize_key( $post_id,60 ),1000*( absint( time()/1000 ) ) );
		eos_dp_pro_call_gtmetrix( $post_id,$post_title,$url_to_test,$user_name,$api_key );
		die();
		exit;
	}
	else{
		esc_html_e( 'The page you want to test does not exist!','eos-dp-pro' );
		die();
		exit;
	}
	echo 0;
	die();
}
add_action( 'wp_ajax_eos_dp_pro_gpsi_test', 'eos_dp_pro_gpsi_test' );
//Google PSI test
function eos_dp_pro_gpsi_test(){
	eos_dp_pro_check_intentions_and_rights( 'eos_dp_pro_gpsi_test' );
	if( isset( $_POST['post_id'] ) || ( isset( $_POST['post_type'] ) && isset( $_POST['archive_url'] ) ) ){
		if( isset( $_POST['post_id'] ) ){
			$post_id = absint( $_POST['post_id'] );
			$post = get_post( $post_id );
			$post_title = is_object( $post ) ? $post->post_title : '';
			$page_url = get_permalink( absint( $post_id ) );
		}
		else{
			$post_id = esc_attr( $_POST['post_type'] );
			$labels = get_post_type_labels( get_post_type_object( $post_id ) );
			$post_type_label = isset( $labels->name ) ? $labels->name : $post_id;
			$post_title = sanitize_text_field( sprintf( __( '%s Archive','eos-dp-pro' ),$post_type_label ) );
			$page_url = esc_url( $_POST['archive_url'] );
		}
		$dpOpts = get_site_option( 'fdp_pro_admin' );
		$api_key = esc_attr( $dpOpts['gpsi_api'] );
		$test_id = time();

		$theme = isset( $_POST['theme'] ) && 'false' !== $_POST['theme'] ? '&theme='.esc_attr( $_POST['theme'] ) : '';
		$url_to_test = $page_url.urlencode( '?eos_dp_preview=true&tool=gpsi&fdp_post_id='.$post_id.'&test_id='.$test_id.$theme );
		$device = isset( $_POST['device'] ) ? sanitize_key( $_POST['device'] ) : 'mobile';
		$url = add_query_arg(
				array(
					'url' => esc_url( $url_to_test ),
					'key' => $api_key,
					'strategy' => $device,
				),
				'https://pagespeedonline.googleapis.com/pagespeedonline/v5/runPagespeed'
		);
		$transient = get_transient( 'fdp_gpsi_'.$api_key,'testing' );
		if( $transient && 'testing' === $transient ){
			esc_html_e( 'Too many requests. Please wait for a few seconds before to try again.','eos-dp-pro' );
			die();
			exit;
		}
		else{
			if( isset( $_POST['plugins_path'] ) ){
				set_transient( 'fdp_test_gpsi_'.sanitize_key( $post_id ),sanitize_text_field( $_POST['plugins_path'] ),60 );
			}
			set_transient( 'fdp_testing_nonce_'.sanitize_key( $post_id,60 ),1000*( absint( time()/1000 ) ) );
			set_transient( 'fdp_gpsi_'.$api_key,30 );
		}
		$response = wp_remote_get( $url,array( 'timeout' => 30 ) );
		if( $response && !is_wp_error( $response ) ){
			$body = wp_remote_retrieve_body( $response );
			if( $body ){
				$results = json_decode( $body,true );
				if( isset( $results['lighthouseResult'] ) ){
					$timestamp = $results['analysisUTCTimestamp'];
					$results = $results['lighthouseResult'];
					if( isset( $results['audits'] ) ){
						$disabled_plugins = array_filter( array_unique( explode( ';pn:',sanitize_text_field( $_POST['plugins_path'] ) ) ) );
						if( !empty( $disabled_plugins ) ){
								$disabled_plugins = sanitize_text_field( implode( ',',array_map( 'eos_dp_name_by_path',$disabled_plugins ) ) );
						}
						$values = array(
							'timestamp' => $timestamp,
							'post_id' => $post_id,
							'post_title' => esc_html( $post_title ),
							'page_url' => $page_url,
							'disabled_plugins' => $disabled_plugins,
							'theme' => sanitize_text_field( str_replace( '&theme=','',$theme ) ),
							'device' => $device,
							'message' => esc_html__( 'Results added to the reports.','eos-dp-pro' )
						);
						$values2 = array(
							'cls' => round( $results['audits']['cumulative-layout-shift']['numericValue'],3 ),
							'si' => round( $results['audits']['speed-index']['numericValue'],1 ),
							'fcp' => round( $results['audits']['first-contentful-paint']['numericValue'],1 ),
							'lcp' => round( $results['audits']['largest-contentful-paint']['numericValue'],1 ),
							'tbt' => round( $results['audits']['total-blocking-time']['numericValue'],1 ),
							'fmp' => round( $results['audits']['first-meaningful-paint']['numericValue'],1 ),
							'srt' => round( $results['audits']['server-response-time']['numericValue'],1 ),
							't' => round( $results['timing']['total'],1 ),
							'score' => 100 * ( $results['categories']['performance']['score'] ),
							'tool' => 'gpsi',
							'test_id' => $test_id
						);
						$values = array_merge( $values,$values2 );
						$writeAccess = false;
						$access_type = get_filesystem_method();
						if( $access_type === 'direct' ){
							$creds = request_filesystem_credentials( EOS_WP_ADMIN,'',false,false,array() );
							if ( ! WP_Filesystem( $creds ) ) {
								return false;
							}
							global $wp_filesystem;
							$writeAccess = true;
						}
						if( true === $writeAccess ){
							eos_dp_pro_write_json_from_array( $values,true,'gpsi' );
						}
						$screenshot = $results['audits']['final-screenshot']['details']['data'];
						$values2 = array_merge( $values2,array( 'screenshot' => esc_attr( $screenshot ) ) );
						echo wp_json_encode( array_map( 'sanitize_text_field',$values2 ) );
						die();
						exit;
					}
				}
			}
		}
	}
	else{
		esc_html_e( 'The page you want to test doesn\' exist!','eos-dp-pro' );
		die();
		exit;
	}
	echo 0;
	die();
}
add_action( 'wp_ajax_eos_dp_pro_remove_test','eos_dp_pro_remove_test' );
//Remove test from report
function eos_dp_pro_remove_test(){
	if( !isset( $_REQUEST['test_id'] ) ){
		die();
		exit;
	}
	eos_dp_pro_check_intentions_and_rights( 'eos_dp_pro_remove_test' );
	$tool = isset( $_REQUEST['tool'] ) ? sanitize_key( $_REQUEST['tool'] ) : 'gtmetrix';
	$test_id =  sanitize_key( $_REQUEST['test_id'] );
	$tool_key = 'last_'.$tool;
	$writeAccess = false;
	$access_type = get_filesystem_method();
	if( $access_type === 'direct' ){
		$creds = request_filesystem_credentials( EOS_WP_ADMIN, '', false, false, array() );
		if ( ! WP_Filesystem( $creds ) ) {
			return false;
		}
		global $wp_filesystem;
		$writeAccess = true;
	}
	$upload_dir = wp_upload_dir();
	if( true === $writeAccess ){
		$arr = array();
		$json = false;
		$opts = eos_dp_pro_get_option( 'fdp_pro_admin' );
		$last_tool = isset( $opts[$tool_key] ) ? sanitize_title( $opts[$tool_key] ) : md5( time() );
		if( file_exists( $upload_dir['basedir'].'/FDP/reports/'.$tool.'/fdp-report-'.$last_tool.'.json' ) ){
			$json = $wp_filesystem->get_contents( $upload_dir['basedir'].'/FDP/reports/'.$tool.'/fdp-report-'.$last_tool.'.json' );
		}
		else{
			die();
			exit;
		}
		if( $json && '' !== $json ){
			$arr = json_decode( $json,true );
		}
		foreach( $arr as $k => $arr2 ){
			if( isset( $arr2['test_id'] ) && (string) sanitize_key( $test_id ) === (string) sanitize_key( $arr2['test_id'] ) ){
				krsort( $arr );
				$arr = array_slice( $arr,0,100,true );
				unset( $arr[$k] );
				eos_dp_pro_write_json_from_array( $arr,false,$tool );
				echo 1;
				die();
				exit;
			}
		}
		die();
		exit;
	}
}
add_action( 'wp_ajax_eos_dp_pro_save_report_table', 'eos_dp_pro_save_report_table' );
//Save report table
function eos_dp_pro_save_report_table(){
	eos_dp_pro_check_intentions_and_rights( 'eos_dp_pro_report' );
	$tool = isset( $_REQUEST['tool'] ) ? sanitize_key( $_REQUEST['tool'] ) : 'gtmetrix';
	$tool_key = 'last_'.$tool;
	$writeAccess = false;
	$access_type = get_filesystem_method();
	if( $access_type === 'direct' ){
		$creds = request_filesystem_credentials( EOS_WP_ADMIN, '', false, false, array() );
		if ( ! WP_Filesystem( $creds ) ) {
			return false;
		}
		global $wp_filesystem;
		$writeAccess = true;
	}
	$upload_dir = wp_upload_dir();
	if( true === $writeAccess ){
		$arr = array();
		$json = false;
		$opts = eos_dp_pro_get_option( 'fdp_pro_admin' );
		$last_tool = isset( $opts[$tool_key] ) ? sanitize_title( $opts[$tool_key] ) : md5( time() );
		if( file_exists( $upload_dir['basedir'].'/FDP/reports/'.$tool.'/fdp-report-'.$last_tool.'.json' ) ){
			$json = $wp_filesystem->get_contents( $upload_dir['basedir'].'/FDP/reports/'.$tool.'/fdp-report-'.$last_tool.'.json' );
		}
		else{
			die();
			exit;
		}
		if( $json && '' !== $json ){
			$arr = json_decode( $json,true );
		}
		krsort( $arr );
		$arr = array_slice( $arr,0,100,true );
		if( isset( $_POST['rows'] ) ){
			$rows = json_decode( stripslashes( $_POST['rows'] ),true );
			$arr2 = array();
			foreach( $rows as $row ){
				if( isset( $arr[$row] ) ){
					$arr2[sanitize_key( $row )] = $arr[$row];
				}
			}
			$arr = $arr2;
			eos_dp_pro_write_json_from_array( $arr,false,$tool );

		}
		$title = sprintf( __( '%s, Speed tests results','eos-dp-pro' ),get_bloginfo( 'name' ) );
		require_once EOS_DP_PRO_PLUGIN_DIR.'/admin/templates/pages/fdp-report.php';
		ob_start();
		eos_dp_report_table( $arr );
		$table = ob_get_clean();
		$wp_head = '<title>'.esc_html( $title ).'</title>';
		$wp_head .= '<link href="https://fonts.googleapis.com/css?family=Quicksand&display=swap" rel="stylesheet">';
		$wp_head .= '<style type="text/css">';
		$wp_head .= 'a{text-decoration:none;color:inherit}h1,h2{text-align:center;margin-top:32px}::selection{color:#ffffff;background:#404040}body{font-family:Quicksand,sans-serif}.dashicons-admin-links:before{content: "link"}.alternate{background:#f9f9f9}table,tr,td,th{font-size:13px;border: 1px solid #929eaa;text-align:center;padding:2px}';
		$wp_head .= '</style>';
		$html = '<html><head>'.$wp_head.'</head><body><h1>'.esc_html( get_bloginfo( 'name' ) ).'</h1><h2>'.esc_html__( 'Speed test results','eos-dp-pro' ).'</h2><div style="max-width:1300px;margin:0 auto;padding:20px">'.$table.'</div></body></html>';
		$wp_filesystem->put_contents( $upload_dir['basedir'].'/FDP/reports/'.$tool.'/fdp-report-'.$last_tool.'.html',$html );
		echo 1;
		die();
		exit;
	}
	echo 0;
	die();
}

add_action( 'wp_ajax_eos_dp_pro_global_hooks_delete_row', 'eos_dp_pro_global_hooks_delete_row' );
//Save hooks to remove
function eos_dp_pro_global_hooks_delete_row(){
	if( !isset( $_POST['hook_name'] ) || !isset( $_POST['function_name'] ) || !isset( $_POST['post_id'] ) ){
		die();
		exit;
	}
	eos_dp_pro_check_intentions_and_rights( 'eos_dp_global_hooks_nonce' );
	$opts = eos_dp_pro_get_option( 'eos_dp_pro_main' );
	$hooks_opts = $opts['fdp_hooks'];
	$opts_post = json_decode( sanitize_text_field( stripslashes( $hooks_opts[absint( $_POST['post_id'] )] ) ),true );
	$unsets = array();
	if( isset( $opts_post[ sanitize_key( $_POST['hook_name'] )] ) ){
		$opts_post_hook_name = $opts_post[sanitize_key( $_POST['hook_name'] )];
		foreach( $opts_post_hook_name as $n => $arr ){
			if( $_POST['function_name'] === $arr[0] ){
				$unsets[] = $n;
			}
		}
	}
	foreach( $unsets as $k ){
		unset( $opts_post_hook_name[$k] );
	}
	$opts_post[sanitize_key( $_POST['hook_name'] )] = $opts_post_hook_name;
	$hooks_opts[absint( $_POST['post_id'] )] = wp_json_encode( $opts_post );
	$opts['fdp_hooks'] = $hooks_opts;
	echo eos_dp_pro_update_option( 'eos_dp_pro_main',$opts );
	die();
	exit;
}
add_action( 'wp_ajax_eos_dp_pro_save_translation_urls', 'eos_dp_pro_save_translation_urls' );
//Save Translation URLs
function eos_dp_pro_save_translation_urls(){
	eos_dp_pro_check_intentions_and_rights( 'eos_dp_pro_save_translation_urls' );
	if( !isset( $_POST['setts'] ) ){
		die();
		exit;
	}
	$setts = str_replace(
		array( '/',PHP_EOL,'\r\n','\n\r','\n','\r',' ,',', ',' ',',,',';','|' ),
		array( '',',',',',',',',',',',',',',','',',',',',',' ),
		sanitize_text_field( $_POST['setts'] )
	);
	$opts = eos_dp_get_option( 'eos_dp_pro_main' );
	if( !$opts || empty( $opts ) ){
		$opts = array();
	}
	if( isset( $opts['translation_urls'] ) && $opts['translation_urls'] === sanitize_text_field( $setts ) ){
		echo 1;
		die();
		exit;
	}
	$opts['translation_urls'] = sanitize_text_field( $setts );
	echo eos_dp_update_option( 'eos_dp_pro_main',$opts ) ? 1 : '';
	die();
	exit;
}

add_action( 'wp_ajax_eos_dp_pro_save_hooks', 'eos_dp_pro_save_hooks' );
//Save hooks to remove
function eos_dp_pro_save_hooks(){
	eos_dp_pro_check_intentions_and_rights( 'fdp-save-hooks' );
	if( !isset( $_POST['removed_hooks'] ) || !isset( $_POST['post_id'] ) ){
		die();
		exit;
	}
	$opts = eos_dp_pro_get_option( 'eos_dp_pro_main' );
	$hooks_opts = $opts['fdp_hooks'];
	$hooks_opts[absint( $_POST['post_id'] )] = sanitize_text_field( stripslashes( $_POST['removed_hooks'] ) );
	$opts['fdp_hooks'] = $hooks_opts;
	eos_dp_pro_update_option( 'eos_dp_pro_main',$opts );
	die();
	exit;
}
//Check for intentions and rights
function eos_dp_pro_check_intentions_and_rights( $nonce_action ){
	$nonce = isset( $_POST['nonce'] ) ? $_POST['nonce'] : false;
	if (
		false === $nonce
		|| ! wp_verify_nonce( sanitize_text_field( $nonce ),$nonce_action ) //check for intentions
		|| !current_user_can( 'activate_plugins' ) //check for rights
	) {
	   echo 0;
	   die();
	   exit;
	}
}

//Update json file from array
function eos_dp_pro_write_json_from_array( $arr,$read_json = true,$tool = 'gtmetrix' ){
	$writeAccess = false;
	$access_type = get_filesystem_method();
	if( $access_type === 'direct' ){
		$creds = request_filesystem_credentials( EOS_WP_ADMIN, '', false, false, array() );
		if ( ! WP_Filesystem( $creds ) ) {
			return false;
		}
	}
	global $wp_filesystem;
	$writeAccess = true;
	$upload_dir = wp_upload_dir();
	$opts = eos_dp_pro_get_option( 'fdp_pro_admin' );
	$tool = sanitize_key( $tool );
	$key_tool = 'last_'.$tool;
	$last_tool = isset( $opts[$key_tool] ) ? sanitize_title( $opts[$key_tool] ) : md5( time() );
	if( !$wp_filesystem->is_dir( $upload_dir['basedir'].'/FDP' ) ){
		$wp_filesystem->mkdir( $upload_dir['basedir'].'/FDP' );
	}
	if( !$wp_filesystem->is_dir( $upload_dir['basedir'].'/FDP/reports' ) ){
		$wp_filesystem->mkdir( $upload_dir['basedir'].'/FDP/reports' );
	}
	if( !$wp_filesystem->is_dir( $upload_dir['basedir'].'/FDP/reports/'.$tool ) ){
		$wp_filesystem->mkdir( $upload_dir['basedir'].'/FDP/reports/'.$tool );
	}
	if( !file_exists( $upload_dir['basedir'].'/FDP/index.html' ) ){
		$wp_filesystem->put_contents( $upload_dir['basedir'].'/FDP/index.html','' );
	}
	if( !file_exists( $upload_dir['basedir'].'/FDP/reports/index.php' ) ){
		$wp_filesystem->put_contents( $upload_dir['basedir'].'/FDP/reports/index.html','' );
	}
	if( !file_exists( $upload_dir['basedir'].'/FDP/reports/'.$tool.'/index.php' ) ){
		$wp_filesystem->put_contents( $upload_dir['basedir'].'/FDP/reports/'.$tool.'/index.html','' );
	}
	$jsonA = array();
	if( $read_json && file_exists( $upload_dir['basedir'].'/FDP/reports/'.$tool.'/fdp-report-'.$last_tool.'.json' ) ){
		$json = $wp_filesystem->get_contents( $upload_dir['basedir'].'/FDP/reports/'.$tool.'/fdp-report-'.$last_tool.'.json' );
		if( $json && '' !== $json ){
			$jsonA = json_decode( $json,true );
		}
	}
	$opts[$key_tool] = $last_tool;
	$opts['last_save'] = time();
	update_site_option( 'fdp_pro_admin',$opts,true );
	if( $read_json ){
		$jsonA[time()] = $arr;
		krsort( $jsonA );
	}
	else{
		$jsonA = $arr;
	}
	$jsonA = array_slice( $jsonA,0,100,true );
	$json = json_encode( $jsonA );
	$wp_filesystem->put_contents( $upload_dir['basedir'].'/FDP/reports/'.$tool.'/fdp-report-'.$last_tool.'.json',$json );
}

add_action( 'wp_ajax_eos_dp_pro_export_options','eos_dp_pro_export_options' );
//Export FDP options
function eos_dp_pro_export_options(){
	if( !current_user_can( 'activate_plugins' ) || !isset( $_POST['data'] ) ) return;
	$data = json_decode( sanitize_text_field( stripslashes( $_POST['data'] ) ) );
	if( !isset( $data->nonce ) || !wp_verify_nonce( esc_attr( $data->nonce ),'fdp_migr_export_nonce' ) ){
		die();
		exit;
	}
	$writeAccess = false;
	$access_type = get_filesystem_method();
	if( $access_type === 'direct' && isset( $data->keys ) ){
		$admin_url = is_multisite() ? network_admin_url() : admin_url();
		$creds = request_filesystem_credentials( $admin_url, '', false, false, array() );
		if ( ! WP_Filesystem( $creds ) ) {
			die();
			exit;
		}
		$opts = array();
		$opts_keys = $data->keys;
		$special_keys = array( 'eos_dp_archives','eos_dp_singles' );
		$user = wp_get_current_user();
		$opts['migration_from'] = sanitize_text_field( admin_url() );
		if( in_array( 'eos_dp_singles',$opts_keys ) ){
			global $wpdb;
			$post_types = array_keys( array_merge( array( 'page' => 'page' ),get_post_types( array( 'publicly_queryable' => true ) ) ) );
			$post_ids = implode( ',',get_posts( array(
				'posts_per_page' => -1,
				'fields' => 'ids',
				'post_type' => $post_types,
				'meta_query' => array(
					array(
						'key' => '_eos_deactive_plugins_key',
						'value'   => '',
						'compare' => '!='
					)
				)
			) ) );
			$sql = "SELECT meta_key, meta_value, post_id FROM $wpdb->postmeta WHERE post_id IN ($post_ids) AND meta_value <> ''";
			$postmeta = $wpdb->get_results( $sql,OBJECT );
			$singles_options = array();
			foreach( $postmeta as $i => $obj ){
				if( '_eos_deactive_plugins_key' === $obj->meta_key ){
					$singles_options[sanitize_key( $obj->post_id )] = sanitize_text_field( $obj->meta_value );
				}
			}
			$opts['singles'] = $singles_options;
		}
		foreach( $opts_keys as $key ){
			if( !in_array( $key,$special_keys ) ){
				$opts[$key] = eos_dp_get_option( sanitize_key( $key ) );
			}
		}
		if( in_array( 'eos_dp_archives',$opts_keys ) ){
			$db_opts = eos_dp_get_option( 'eos_dp_opts' );
			$skip_db = ( isset( $db_opts['skip_db_for_archives'] ) && 'true' ===  $db_opts['skip_db_for_archives'] ) || ( defined( 'FDP_SKIP_DB_FOR_ARCHIVES' ) && FDP_SKIP_DB_FOR_ARCHIVES );
			if( $skip_db ){
				$upload_dirs = wp_upload_dir();
				$opts_file = $upload_dirs['basedir'].'/FDP/fdp-options/'.sanitize_key( substr( md5( 'eos_dp_archives' ),0,8 ) ).'.json';
				if( file_exists( $opts_file ) ){
					$archives = json_decode( file_get_contents( $opts_file ) );
				}
			}
			else{
				$archives = eos_dp_get_option( 'eos_dp_archives' );
			}
			$home_urlA = explode( '//',get_home_url() );
			$pref = str_replace( '/','__',$home_urlA[1] );
			$archives_new = array();
			if( isset( $home_urlA[1] ) ){
				foreach( $archives as $key => $value ){
					$key_new = str_replace( $pref,'__home_url__',$key );
					$archives_new[sanitize_key( $key_new )] = sanitize_text_field( $value );
				}
			}
			$opts['eos_dp_archives'] = $archives_new;
		}
		global $wp_filesystem;
		$upload_dir = wp_upload_dir();
		$arrFiles = glob( $upload_dir['basedir'].'/fdp-options-*.txt' );
		foreach( $arrFiles as $file ){
			$wp_filesystem->delete( $file );
		}
		$wp_filesystem->put_contents( $upload_dir['basedir'].'/FDP-options-'.sanitize_title( get_bloginfo( 'name' ) ).'-'.sanitize_text_field( $data->time ).'.txt', base64_encode( serialize( $opts ) ) );
		echo 1;
		die();
		exit;
	}
}

add_action( 'wp_ajax_eos_dp_pro_save_clean_css','eos_dp_pro_save_clean_css' );
//Save reduced CSS
function eos_dp_pro_save_clean_css(){
	if(
		!current_user_can( 'activate_plugins' )
		|| !isset( $_POST['nonce'] )
		|| !isset( $_POST['css_id'] )
		|| !isset( $_POST['post_id'] )
		|| !isset( $_POST['css'] )
		|| !wp_verify_nonce( esc_attr( $_POST['nonce'] ),'fdp_pro_reduce_css' )
	){
		die();
		exit;
	}
	if ( preg_match( '#</?\w+#',$css ) ) {
		echo 'css_not_valid';
		die();
		exit;
	}
	$css = strip_tags( eos_dp_pro_compress_css( stripslashes( $_POST['css'] ) ) );
	echo eos_dp_pro_write_asset_file( esc_attr( $_POST['css_id'] ),$css,absint( $_POST['post_id'] ),'css' );
	die();
	exit;
}
add_action( 'wp_ajax_eos_dp_pro_preview_assets','eos_dp_pro_preview_assets' );
//Preview assets
function eos_dp_pro_preview_assets(){
	eos_dp_pro_save_assets( true );
}
add_action( 'wp_ajax_eos_dp_pro_save_assets','eos_dp_pro_save_assets' );
//Save assets
function eos_dp_pro_save_assets( $preview = false ){
	$nonce_action = $preview ? 'fdp-preview-assets' : 'fdp-save-assets';
	if(
		!current_user_can( 'activate_plugins' )
		|| !isset( $_POST['nonce'] )
		|| !isset( $_POST['asset'] )
		|| !isset( $_POST['assets'] )
		|| !isset( $_POST['assets_active'] )
		|| !in_array( $_POST['asset'],array( 'styles','scripts' ) )
		|| !wp_verify_nonce( esc_attr( $_POST['nonce'] ),$nonce_action )
	){
		die();
		exit;
	}
	$asset_type = sanitize_key( $_POST['asset'] );
	$opts = eos_dp_pro_get_option( 'eos_dp_pro_main' );
	$assets = json_decode( sanitize_text_field( stripslashes( $_POST['assets'] ) ),true );
	$assets_active = json_decode( sanitize_text_field( stripslashes( $_POST['assets_active'] ) ),true );
	$opts_assets_string = $opts[$asset_type];
	$opts_assets = isset( $opts[$asset_type] ) && is_string( $opts[$asset_type] ) ? json_decode( sanitize_text_field( stripslashes( $opts[$asset_type] ) ),true ) : array();
	foreach( $assets_active as $handle => &$arr ){
		foreach( $arr as $k => $v ){
			//$k is e.g. single, post_type, archives, $v is a stringified array of values
			if( $v && false !== strpos( $v,',' ) ){
				$opts_values_string = eos_dp_pro_isset_multi( $opts_assets,array( $handle,$k ) );
				$values_active = explode( ',',rtrim( $v,',' ) );
				if( $opts_values_string && is_string( $opts_values_string ) ){
					$opts_values = explode( ',',rtrim( $opts_values_string,',' ) );
					foreach( $opts_values as $opt_value ){
						if( $opt_value && '' !== $opt_value && in_array( $opt_value,$values_active ) ){
							unset( $opts_values[array_search( $opt_value,$opts_values )] );
						}
					}
					$opts_assets[$handle][$k] = is_array( $opts_values ) ? sanitize_text_field( implode( ',',$opts_values ).',' ) : '';
				}
			}
		}
	}
	foreach( $assets as $handle => $arr ){
		foreach( $arr as $where_type => $values_string ){
			$values = explode( ',',ltrim( rtrim( $values_string,',' ),',' ) );
			if( !isset( $opts_assets[$handle] ) ){
				$opts_assets[sanitize_key( $handle )] = array();
			}
			if( !isset( $opts_assets[$handle][$where_type] ) ){
				$opts_assets[$handle][$where_type] = sanitize_text_field( implode( ',',$values ) );
			}
			else{
				$where_values = eos_dp_pro_isset_multi( $opts_assets,array( $handle,$where_type ) );
				if( $where_values ){
					$opts_values = '' !== $opts_assets[$handle][$where_type] ? explode( ',',rtrim( $opts_assets[$handle][$where_type],',' ) ) : array();
					$opts_assets[$handle][$where_type] = implode( ',',array_filter( array_unique( array_merge( $opts_values,array_map( 'sanitize_text_field',$values ) ) ) ) );
				}
			}
		}
	}
	if( isset( $_POST['singles_active'] ) && '' !== $_POST['singles_active'] && isset( $_POST['post_id'] ) &&  absint( $_POST['post_id'] ) > 0 ){
		if( $preview ){
			set_site_transient( 'fdp_preview_'.get_current_user_id().'_'.sanitize_key( $_POST['action_id'].'_'.absint( $_POST['post_id'] ),'_fdp_'.$asset_type.'_active' ),sanitize_text_field( $_POST['singles_active'] ),60 );
		}
		else{
			update_post_meta( absint( $_POST['post_id'] ),'_fdp_'.$asset_type.'_active',sanitize_text_field( $_POST['singles_active'] ) );
		}
	}
	if( isset( $_POST['moved_to_footer'] ) && '' !== $_POST['moved_to_footer'] ){
		$update_moved_head = false;
		$moved_footer = explode( ',',sanitize_text_field( $_POST['moved_to_footer'] ) );
		$moved_opts = isset( $opts_assets['moved_footer'] ) ? $opts_assets['moved_footer'] : array();
		$moved_opts[sanitize_key( $_POST['post_id'] )] = array_map( 'esc_attr',$moved_footer );
		$opts_assets['moved_footer'] = $moved_opts;
		if( isset( $opts_assets['moved_head'] ) && is_array( $opts_assets['moved_head'] ) ){
			foreach( $opts_assets['moved_head'] as $post_id => $moved_handles ){
				$opts_moved_head = eos_dp_pro_isset_multi( $opts_assets,array( 'moved_head',sanitize_key( $_POST['post_id'] ) ) );
				if( $opts_moved_head ){
					foreach( $moved_handles as $moved_handle ){
						if( in_array( $moved_handle,$opts_moved_head ) ){
							unset( $opts_moved_head[array_search( $moved_handle,$opts_moved_head )] );
							$update_moved_head = true;
						}
					}
				}
			}
			if( $update_moved_head ){
				$opts_assets['moved_head'] = $opts_moved_head;
			}
		}
	}
	if( isset( $_POST['moved_to_head'] ) && '' !== $_POST['moved_to_head'] ){
		$update_moved_footer = false;
		$moved_head = explode( ',',sanitize_text_field( $_POST['moved_to_head'] ) );
		$moved_opts = isset( $opts_assets['moved_head'] ) ? $opts_assets['moved_head'] : array();
		$moved_opts[sanitize_key( $_POST['post_id'] )] = array_map( 'esc_attr',$moved_head );
		$opts_assets['moved_head'] = $moved_opts;
		if( isset( $opts_assets['moved_footer'] ) && $opts_assets['moved_footer'] ){
			foreach( $opts_assets['moved_footer'] as $post_id => $moved_handles ){
				$opts_moved_footer = eos_dp_pro_isset_multi( $opts_assets,array( 'moved_footer',sanitize_key( $_POST['post_id'] ) ) );
				if( $opts_moved_footer ){
					foreach( $moved_handles as $moved_handle ){
						if( in_array( $moved_handle,$opts_moved_footer ) ){
							unset( $opts_moved_footer[array_search( $moved_handle,$opts_moved_footer )] );
							$update_moved_footer = true;
						}
					}
				}
			}
		}
		if( $update_moved_footer ){
			$opts_assets['moved_footer'] = $opts_moved_footer;
		}
	}
	if( isset( $_POST['post_id'] ) && isset( $_POST['reduced_styles'] ) && 'styles' === $_POST['asset'] ){
		if( '' === $_POST['reduced_styles'] ){
			$reduced_opts[sanitize_key( $_POST['post_id'] )] = array();
		}
		else{
			$reduced_styles = explode( ',',sanitize_text_field( $_POST['reduced_styles'] ) );
			$reduced_opts = isset( $opts_assets['reduced_styles'] ) ? $opts_assets['reduced_styles'] : array();
			$reduced_opts[sanitize_key( $_POST['post_id'] )] = array_map( 'esc_attr',$reduced_styles );
			$opts_assets['reduced_styles'] = $reduced_opts;
		}

	}
	if( isset( $_POST['custom_urls'] ) ){
		$opts_assets['custom_urls'] = sanitize_text_field( $_POST['custom_urls'] );
	}
	$opts[$asset_type] = wp_json_encode( $opts_assets );
	if( isset( $_POST['singles_active'] ) && isset( $_POST['post_id'] ) ){
		if( isset( $_POST['action_id'] ) && $preview ){
			set_site_transient( 'fdp_preview_assets_single_active_'.get_current_user_id().'_'.sanitize_key( $_POST['action_id'] ),sanitize_text_field( $_POST['singles_active'] ),60 );
		}
		else{
			update_post_meta( absint( $_POST['post_id'] ),'_fdp_pro_single_active',sanitize_text_field( $_POST['singles_active'] ) );
		}
	}
	if( !$preview && eos_dp_pro_update_option( 'eos_dp_pro_main',$opts ) ){
		echo 1;
	}
	else{
		if( $preview && isset( $_POST['action_id'] ) ){
			set_site_transient( 'fdp_preview_assets_'.get_current_user_id().'_'.sanitize_key( $_POST['action_id'] ),$opts,60 );
			echo 'preview';
		}
	}
	die();
	exit;
}

add_action( 'wp_ajax_eos_dp_pro_import_options','eos_dp_pro_import_options' );
//Import FDP options
function eos_dp_pro_import_options(){
	if( !current_user_can( 'activate_plugins' ) || !isset( $_POST['data'] ) ) return;
	$data = json_decode( stripslashes( $_POST['data'] ) );
	if( !isset( $data->nonce ) || !wp_verify_nonce( esc_attr( $data->nonce ),'fdp_migr_import_nonce' ) ){
		echo esc_html__( 'Are you sure you wanted to upload the FDP options?','eos-dp-pro' );
		die();
		exit;
	}
	if( isset($_FILES['files'] ) ){
		$max_exec_time = ini_get( 'max_execution_time' );
		ini_set( 'max_execution_time',500 );
		$files = $_FILES['files'];
		if( isset( $files['tmp_name'] ) ){
			$writeAccess = false;
			$access_type = get_filesystem_method();
			if( $access_type === 'direct' ){
				$admin_url = is_multisite() ? network_admin_url() : admin_url();
				$creds = request_filesystem_credentials( $admin_url, '', false, false, array() );
				if ( ! WP_Filesystem( $creds ) ){
					echo esc_html__( 'You are not allowed to upload files.','eos-dp-pro' );
					die();
					exit;
				}
				$file_name = $files['name'][0];
				$file_tmp = $files['tmp_name'][0];
				$file_type = $files['type'][0];
				$file_size = $files['size'][0];
				$upload_dir = wp_upload_dir();
				if( $file_size > 3000000 ){
					echo esc_html__( 'File size exceded','eos-dp-pro' );
					die();
					exit;
				}
				$wp_filetype = wp_check_filetype( $file_name );
				$file_ext = $wp_filetype['ext'];
				if( !in_array( $file_ext,array( 'txt' ) ) ){
					printf( esc_html__( 'File extension %s not allowed. Allowed only .txt files.','eos-dp-pro' ),$file_size );
					die();
					exit;
				}
				$file = array(
					'name' => $file_name,
					'type' => $file_type,
					'tmp_name' => $file_tmp,
					'error' => $files['error'][0],
					'size' => $file_size
				);
				$movefile = wp_handle_upload( $file,array( 'test_form' => false ) );
				global $wpdb;
				global $wp_filesystem;
				$file_content = $wp_filesystem->get_contents( $movefile['file'] );
				$opts = unserialize( base64_decode( $file_content ) );
				if( $opts ){
					$opts_keys = $data->keys;
					$special_keys = array( 'eos_dp_archives','eos_dp_singles' );
					foreach( $opts_keys as $key ){
						if( !in_array( $key,$special_keys ) && isset( $opts[$key] ) ){
							eos_dp_update_option( $key,json_decode( sanitize_text_field( json_encode( $opts[$key] ) ),true ) );
						}
					}
					if( in_array( 'eos_dp_singles',$opts_keys ) && isset( $opts['singles'] ) ){
						$singles = json_decode( sanitize_text_field( json_encode( $opts['singles'] ) ),true );
						$post_ids = implode( ',',array_keys( $singles ) );
						$conditions = '';
						// foreach( $singles as $id => $v ){
						// 	$conditions .= 'WHEN post_id='.sanitize_text_field( $id ).' THEN \''.sanitize_text_field( $v ).'\'';
						// }
						// $sql = "UPDATE $wpdb->postmeta SET meta_value = (CASE $conditions END) WHERE post_id IN ($post_ids) AND meta_key='_eos_deactive_plugins_key';";
						// $results = $wpdb -> query( $sql );
						// if( $results ){
							foreach( $singles as $id => $v ){
								$url = get_permalink( $id );
								$path = str_replace( get_home_url(),'',$url );
								eos_dp_update_url_options( $path,$id,$v,get_post_type( $id ) );
								update_post_meta( $id,'_eos_deactive_plugins_key',$v );
							}
						// }
					}
					if( in_array( 'eos_dp_archives',$opts_keys ) && isset( $opts['eos_dp_archives'] ) ){
						$archives = $opts['eos_dp_archives'];
						$home_urlA = explode( '//',get_home_url() );
						$pref = str_replace( '/','__',$home_urlA[1] );
						$archives_new = array();
						if( isset( $home_urlA[1] ) ){
							foreach( $archives as $key => $value ){
								$key_new = str_replace( '__home_url__',$pref,$key );
								$archives_new[sanitize_key( $key_new )] = sanitize_text_field( $value );
								$key_newArr = explode( '__',$key_new );
								unset( $key_newArr[0] );
								eos_dp_update_url_options( implode( '/',$key_newArr ),'archive',sanitize_text_field( $value ),$key_newArr[count( $key_newArr )] );
							}
						}
						if( !defined( 'FDP_SKIP_DB_FOR_ARCHIVES' ) || !FDP_SKIP_DB_FOR_ARCHIVES ){
							$opts = eos_dp_get_option( 'eos_dp_opts' );
							$skip_db = isset( $opts['filesystem_last_status'] ) && 'ok' === $opts['filesystem_last_status'] && isset( $opts['skip_db_for_archives'] ) && 'true' === $opts['skip_db_for_archives'];
							if( !$skip_dp ){
								eos_dp_update_option( 'eos_dp_archives',$archives_new );
							}
						}
					}
					echo 1;
					die();
					exit;
				}
			}
		}
		ini_set( 'max_execution_time',$max_exec_time );
	}
	esc_html_e( 'Something went wrong importing the file','eos-dp-pro' );
	die();
	exit;
}

add_action( 'wp_ajax_eos_dp_pro_hooks_recorder_start','eos_dp_pro_hooks_recorder_start' );
//Recorder start
function eos_dp_pro_hooks_recorder_start(){
  eos_dp_pro_check_intentions_and_rights( 'eos_dp_pro_hooks_recorder_start_nonce' );
	echo set_site_transient( 'eos_dp_pro_recording_id_'.get_current_user_id(),sanitize_key( $_POST['rec_id'] ),60*10 );
	die();
	exit;
}

add_action( 'wp_ajax_eos_dp_pro_hooks_recorder_stop','eos_dp_pro_hooks_recorder_stop' );
//Recorder stop
function eos_dp_pro_hooks_recorder_stop(){
  if( !isset( $_POST['rec_id'] ) ) {
	   echo 0;
	   die();
	   exit;
	}
	eos_dp_pro_check_intentions_and_rights( 'eos_dp_pro_hooks_recorder_stop_nonce' );
	$rec_id = sanitize_key( $_POST['rec_id'] );
	$all_hooks = get_site_transient( 'eos_dp_pro_recording_hooks_'.$rec_id );
	delete_site_transient( 'eos_dp_pro_recording_hooks_'.$rec_id );
  delete_site_transient( 'eos_dp_pro_recording_id_'.get_current_user_id() );
  delete_site_transient( 'eos_dp_pro_recording_uri_'.$rec_id.'_'.get_current_user_id() );
	if( $all_hooks && is_array( $all_hooks ) ){
		echo sanitize_text_field( wp_json_encode( array( 'hooks' => $all_hooks ) ) );
	}
	die();
	exit;
}

add_action( 'wp_ajax_eos_dp_pro_bulk_actions','eos_dp_pro_bulk_actions' );
//Bulk actions on FDP options
function eos_dp_pro_bulk_actions(){
	if( !current_user_can( 'activate_plugins' ) || !isset( $_POST['data'] ) ) return;
	$data = json_decode( sanitize_text_field( stripslashes( $_POST['data'] ) ) );
	do_action( 'fpd_before_bulk_actions',$data );
	if( !isset( $data->nonce ) || !wp_verify_nonce( esc_attr( $data->nonce ),'fdp_bulk_actions_nonce' ) ){
		echo esc_html__( 'Are you sure you wanted to perform a bulk action?','eos-dp-pro' );
		die();
		exit;
	}
	if( 'none' === $data->plugin ){
		echo esc_html__( 'You need to select a plugiin.','eos-dp-pro' );
		die();
		exit;
	}
	if( 'none' === $data->bulk_action ){
		echo esc_html__( 'You need to select an action.','eos-dp-pro' );
		die();
		exit;
	}
	if( empty( $data->options ) ){
		echo esc_html__( 'You need to select at least an option.','eos-dp-pro' );
		die();
		exit;
	}
	if( esc_attr( $data->plugin ) !== $data->plugin ){
		echo esc_html__( 'Plugin not valid','eos-dp-pro' );
		die();
		exit;
	}
	if( !in_array( $data->bulk_action,array( 'activate','deactivate' ) ) ){
		echo esc_html__( 'Action not valid','eos-dp-pro' );
		die();
		exit;
	}
	if( !in_array( $data->plugin,eos_dp_active_plugins() ) ){
		echo esc_html__( "You can't perform a bulk action on a plugin that is not active.","eos-dp-pro" );
		die();
		exit;
	}
	$valid_options = array_keys( eos_dp_pro_options_keys( true ) );
	foreach( $data->options as $option ){
		if( $option && !in_array( $option,$valid_options ) ){
			die();
			exit;
		}
		$transient = get_site_transient( 'fdp_plugin_activated_notification' );
		if( $transient && $data->plugin === $transient ){
			delete_site_transient( 'fdp_plugin_activated_notification' );
		}
		eos_dp_pro_execute_bulk_action( $data->plugin,$data->bulk_action,$option,$data->singles_pt );
	}
	echo 1;
	die();
	exit;
}

//It executs a bulk action given the plugin, the action and the options
function eos_dp_pro_execute_bulk_action( $plugin,$action,$option_key,$singles_post_types ){
	$max_exec_time = ini_get( 'max_execution_time' );
	ini_set( 'max_execution_time',500 );
	if( 'eos_dp_singles' !== $option_key ){
		$opts = eos_dp_pro_get_option( $option_key );
		if( !$opts ){
			if( in_array( $option_key,array( 'eos_dp_pro_ajax_setts','eos_dp_by_url','eos_dp_by_admin_url','eos_dp_admin_setts' ) ) ){
				return;
			}
			$opts = eos_dp_pro_fill_opts_by_key( $option_key );
		}
	}
	switch( $option_key ){
		case 'eos_post_types_plugins':
		case 'eos_dp_by_url':
		case 'eos_dp_by_admin_url':
			$el_idxs = array( 'eos_post_types_plugins' => 1,'eos_dp_by_url' => 'plugins','eos_dp_by_admin_url' => 'plugins' );
			$opts = eos_dp_pro_opts_after_bulk( $opts,$plugin,$action,$el_idxs,$option_key );
			break;
		case 'eos_dp_archives':
			$opts = eos_dp_pro_opts_after_bulk( $opts,$plugin,$action,false,$option_key );
			break;
		case 'eos_dp_admin_setts':
		case 'eos_dp_integration_actions':
			$opts = eos_dp_pro_opts_after_bulk( $opts,$plugin,$action,false,$option_key );
			break;
		case 'eos_dp_mobile':
		case 'eos_dp_desktop':
		case 'eos_dp_search':
			if( 'activate' === $action ){
				if( in_array( $plugin,$opts ) ){
					unset( $opts[array_search( $plugin,$opts )] );
				}
			}
			elseif( 'deactivate' === $action ){
				if( !in_array( $plugin,$opts ) ){
					$opts[] = sanitize_text_field( $plugin );
				}
			}
			break;
		case 'eos_dp_pro_ajax_setts':
			$all_actions = json_decode( str_replace( '\\','',$opts ) );
			$opts = json_encode( eos_dp_pro_opts_after_bulk( $all_actions,$plugin,$action,false,$option_key ) );
			break;
		case 'eos_dp_singles':
			if( empty( $singles_post_types ) ) break;
			$post_types = array_map( 'esc_attr',$singles_post_types );
			$ids = get_posts( array( 'posts_per_page' => -1,'post_type' => $post_types,'post_status' => 'any','fields' => 'ids' ) );
			if( !empty( $ids ) ){
				$meta_new = array();
				$meta = eos_dp_get_multiple_metadata( '_eos_deactive_plugins_key',implode( ',',$ids ) );
				foreach( $meta as $obj ){
					$meta_value = $obj->meta_value;
					$disabled_plugins = '' !== $meta_value ? explode( ',',$meta_value ) : array();
					if( 'activate' === $action ){
						if( in_array( $plugin,$disabled_plugins ) ){
							unset( $disabled_plugins[array_search( $plugin,$disabled_plugins )] );
						}
					}
					elseif( 'deactivate' === $action ){
						if( !in_array( $plugin,$disabled_plugins ) ){
							$disabled_plugins[] = $plugin;
						}
					}
					$meta_new[absint( $obj->post_id )] = sanitize_text_field( implode( ',',$disabled_plugins ) );
				}
				if( !empty( $meta_new ) ){
					global $wpdb;
					$conditions = '';
					foreach( $meta_new as $id => $v ){
						$conditions .= ' WHEN post_id='.sanitize_text_field( $id ).' THEN \''.sanitize_text_field( $v ).'\'';
						$url = get_permalink( $id );
						$path = str_replace( get_home_url(),'',$url );
						eos_dp_update_url_options( $path,$id,$v,get_post_type( $id ) );
					}
					$post_ids = implode( ',',$ids );
					$sql = "UPDATE $wpdb->postmeta SET meta_value = (CASE $conditions END) WHERE post_id IN ($post_ids) AND meta_key='_eos_deactive_plugins_key';";
					$results = $wpdb -> query( $sql );
				}
			}
			break;
	}
	if( 'eos_dp_singles' !== $option_key ){
		eos_dp_pro_update_option( $option_key,$opts );
	}
	ini_set( 'max_execution_time',$max_exec_time );
}

function eos_dp_pro_fill_opts_by_key( $option_key ){
	switch( $option_key ){
		case 'eos_dp_integration_actions':
			$opts = array();
			$plugins_integration = eos_dp_plugins_integration();
			foreach( $plugins_integration as $plug_slug => $arr ){
				foreach( $arr['ajax_actions'] as $action => $arr2 ){
					$opts[$action] =  isset( $arr2['default'] ) && 'disabled' === $arr2['default'] ? $active_plugins : array();
				}
			}
			break;
		case 'eos_dp_mobile':
		case 'eos_dp_desktop':
		case 'eos_dp_search':
			$opts = array();
			break;
		case 'eos_post_types_plugins':
			$opts = apply_filters( 'eos_dp_plugins_table',eos_dp_plugins_table() );
			break;
		case 'eos_dp_by_url';
		case 'eos_dp_by_admin_url':
			$opts = array( array( 'url' => '','plugins' => '' ) );
			break;
		case 'eos_dp_archives';
			$plugins_table = eos_dp_plugins_table();
			$active_plugins = eos_dp_active_plugins();
			$shop_url = false;
			$page_for_posts = eos_dp_get_option( 'page_for_posts' );
			$show_on_front = eos_dp_get_option( 'show_on_front' );
			if( function_exists( 'wc_get_page_id' ) ){
				$shop_url = get_permalink( wc_get_page_id( 'shop' ) );
			}
			foreach( $plugins_table as $post_type => $plugins ){
				$post_type_obj = get_post_type_object( $post_type );
				$archive_url = get_post_type_archive_link( $post_type );
				$skip = false;
				if( is_object( $post_type_obj ) && isset( $post_type_obj->rewrite ) ){
					$rewrite = $post_type_obj->rewrite;
					$slug = isset( $rewrite['slug'] ) ? $rewrite['slug'] : false;
					$page = get_page_by_path( esc_attr( $slug ) );
					if( is_object( $page ) ){
						if( get_permalink( $page->ID ) === $archive_url ){
							$skip = true;
						}
					}
				}
				if( !$skip && $shop_url !== $archive_url && $post_type !== 'page' && $archive_url && ( $post_type !== 'post' || ( !$page_for_posts && 'posts' === $show_on_front ) ) ){
					$archive_url = remove_query_arg( 'lang',$archive_url );
					$kArr = explode( '//',$archive_url );
					if( isset( $kArr[1] ) ){
						$key = $kArr[1];
					}
					$opts[sanitize_key( str_replace( '/','__',rtrim( $key,'/' ) ) )] = array_fill( 0,count( $active_plugins ),',' );
				}
				$taxonomies = get_taxonomies();
				if( $taxonomies && !empty( $taxonomies ) ){
					foreach( $taxonomies as $tax ){
						$terms = get_terms( array( 'taxonomy' => $tax -> name,'hide_empty' => true,'orderby' => 'name','order' => 'ASC' ) );
						$permalink_structure = eos_dp_get_option( 'permalink_structure' );
						$plugins_by_dirs = isset( $GLOBALS['eos_dp_plugins_by_dirs'] ) ? $GLOBALS['eos_dp_plugins_by_dirs'] : eos_dp_get_plugins();
						foreach( $terms as $term ){
							$archive_url = get_term_link( $term );
							if( !is_wp_error( $archive_url ) ){
								$archive_url = remove_query_arg( 'lang',$archive_url );
								$kArr = explode( '//',$archive_url );
								if( isset( $kArr[1] ) ){
									$key = $kArr[1];
								}
								$key = sanitize_key( str_replace( '/','__',rtrim( $key,'/' ) ) );
								if( false !== strpos( $permalink_structure,'%category%' ) && '%postname%' === basename( $permalink_structure )  && '.' === eos_dp_get_option( 'category_base' ) ){
									$key = str_replace( '__.__','__',$key );
								}
								$opts[$key] = array_fill( 0,count( $active_plugins ),',' );
							}
						}
					}
				}
			}
			break;
	}
	return $opts;
}

function eos_dp_pro_opts_after_bulk( $opts,$plugin,$action,$el_idxs = false,$option_key = false ){
	if( $opts ){
		foreach( $opts as $k => &$v ){
			if( $el_idxs && $option_key ){
				$disabled_plugins = isset( $v[$el_idxs[$option_key]] ) && '' !== $v[$el_idxs[$option_key]] ? explode( ',',$v[$el_idxs[$option_key]] ) : array();
			}
			else{
				if( $v && is_array( $v ) ){
					$disabled_plugins = $v;
				}
				else{
					$disabled_plugins = $v && is_string( $v ) && '' !== $v ? explode( ',',$v ) : array();
				}
			}
			$disabled_plugins = array_unique( $disabled_plugins );
			if( 'activate' === $action ){
				if( !empty( $disabled_plugins ) && in_array( $plugin,$disabled_plugins ) ){
					$disabled_plugins[array_search( $plugin,$disabled_plugins )] = '';
				}
			}
			elseif( 'deactivate' === $action ){
				$disabled_plugins[] = $plugin;
			}
			if( $el_idxs && $option_key ){
				$v[$el_idxs[$option_key]] = sanitize_text_field( implode( ',',$disabled_plugins ) );
			}
			else{
				$v = sanitize_text_field( implode( ',',$disabled_plugins ) );
			}
		}
	}
	return $opts;
}
add_action( 'wp_ajax_eos_dp_pro_whois_option','eos_dp_pro_whois_option' );
function eos_dp_pro_whois_option(){
	if( !isset( $_POST['option'] ) ) {
		 echo 0;
		 die();
		 exit;
	}
	eos_dp_pro_check_intentions_and_rights( 'fdp_autoload_nonce' );
	$uniqid = uniqid();
	set_site_transient( 'fdp_autoload_whois_nonce',sanitize_text_field( $uniqid ),20 );
	$args = eos_dp_pro_get_user_headers( array( 'sslverify' => false ),true );
	$response = wp_remote_get(
		add_query_arg( array(
			'action' => 'deactivate',
			'plugins' => 'none',
			'fdp_whois_option' => sanitize_text_field( $_POST['option'] ),
			't' => time(),
			'user_id' => get_current_user_id(),
			'nonce' => $uniqid
			),
			get_home_url()
		),
		$args
	);
	$output = sprintf( esc_html__( 'Visiting the home page it looks the option %s is not called. But it may be called on other pages.','eos-dp-pro' ),sanitize_text_field( $_POST['option'] ) );
	if( $response && !is_wp_error( $response ) ){
		$body = wp_remote_retrieve_body( $response );
		if( false !== strpos( $body,'<div id="fdp-pro-option-files">' ) ){
			$arr = explode( '<div id="fdp-pro-option-files">',$body );
			$arr = explode( '</div>',$arr[1] );
			$files = json_decode( stripslashes( sanitize_text_field( $arr[0] ) ),true );
			if( $files && is_array( $files ) && !empty( $files ) ){
				$output = sprintf( esc_html__( 'Visiting the home page the option %s is called by the following files:','eos-dp-pro' ),sanitize_text_field( $_POST['option'] ) ).PHP_EOL.PHP_EOL;
				$n = 0;
				foreach( $files as $file ){
					$output .= str_replace( ABSPATH,'',$file ).PHP_EOL;
					++$n;
				}
			}
			echo $output;
		}
	}
	die();
	exit;
}
add_action( 'wp_ajax_eos_dp_pro_autoload','eos_dp_pro_autoload' );
function eos_dp_pro_autoload(){
	if( !isset( $_POST['noautoload'] ) ) {
		 echo 0;
		 die();
		 exit;
	}
	eos_dp_pro_check_intentions_and_rights( 'fdp_autoload_nonce' );
	$no_autoloads = json_decode( stripslashes( sanitize_text_field( $_POST['noautoload'] ) ) );
	foreach( $no_autoloads as $option => $size ){
		eos_dp_pro_update_autoload( $option );
	}
	if( isset( $_POST['autoload'] ) ){
		$autoloads = explode( ',',rtrim( stripslashes( sanitize_text_field( $_POST['autoload'] ) ),',' ) );
		foreach( $autoloads as $option ){
			if( !empty( $option ) ){
				eos_dp_pro_update_autoload( $option,'yes' );
			}
		}
	}
	echo eos_dp_pro_update_option( 'eos_dp_noautoload',sanitize_text_field( $_POST['noautoload'] ) ) || eos_dp_pro_get_option( 'eos_dp_noautoload' ) === sanitize_text_field( $_POST['noautoload'] );
	die();
	exit;

}

add_action( 'wp_ajax_eos_dp_pro_get_file_size','eos_dp_pro_get_file_size' );
function eos_dp_pro_get_file_size(){
	if(
		!current_user_can( 'activate_plugins' )
		|| !isset( $_POST['nonce'] )
		|| !isset( $_POST['url'] )
		|| !wp_verify_nonce( esc_attr( $_POST['nonce'] ),'fdp_pro_reduce_css' )
	){
		die();
		exit;
	}
	$siteUrlA = explode( '//',get_site_url() );
	$src = esc_url( $_POST['url'] );
	$srcA = explode( '//',$src );
	$srcA = explode( '?',$srcA[1] );
	$path = str_replace( $siteUrlA[1],ABSPATH,$srcA[0] );
	echo file_exists( $path ) ? esc_html( sprintf( '%s kB',round( filesize( $path )/1024,1 ) ) ) : 'unknown';
	die();
	exit;
}

add_action( 'wp_ajax_eos_dp_pro_reset_assets','eos_dp_pro_reset_assets' );
function eos_dp_pro_reset_assets(){
	if(
		!current_user_can( 'activate_plugins' )
		|| !isset( $_POST['nonce'] )
		|| !isset( $_POST['asset'] )
		|| !wp_verify_nonce( esc_attr( $_POST['nonce'] ),'fdp-reset-assets' )
	){
		die();
		exit;
	}
	$asset_type = sanitize_key( $_POST['asset'] );
	$opts = eos_dp_pro_get_option( 'eos_dp_pro_main' );
	$opts[sanitize_key( $_POST['asset'] )] = array();
	eos_dp_pro_update_option( 'eos_dp_pro_main',$opts );
	echo 1;
	die();
	exit;
}

//Compresses the given CSS, it will strip comments and not useful spaces
function eos_dp_pro_compress_css( $css ){
	if( $css !== '' ){
		$css = preg_replace( '/\/\*(.*?)\*\//','', $css );
		$replace = array(
			"url()" => 'none',
			"  " => " ",
			"  " => " ",
			"/\/\*(.*?)\*\//" => '',
			"/\/\*(?:(?!\*\/)[\s\S])*\*\/|[\r\n\t]+/" => '',
			": "  => ":",
			" :"  => ":",
			"; "  => ";",
			" ;"  => ";",
			" {"  => "{",
			" }"  => "}",
			", "  => ",",
			" ,"  => ",",
			"{ "  => "{",
			";}"  => "}",
			"} "  => "}",
			"!important" => " !important",
			"  !important" => " !important",
		);
		$search = array_keys( $replace );
		$css = str_replace( $search, $replace, $css );
	}
	$css = str_replace( '  ',' ',str_replace( '&nbsp;',' ',$css ) );
	return $css;
}

//Helper function to write file
function eos_dp_pro_write_asset_file( $asset_id,$content,$post_id,$asset_type ){
	if( !function_exists( 'get_filesystem_method' ) ){
		require_once ( ABSPATH . '/wp-admin/includes/file.php' );
		WP_Filesystem();
	}
	$access_type = get_filesystem_method();
	if( $access_type === 'direct' ){
		$creds = request_filesystem_credentials( admin_url(), '', false, false, array() );
		if ( ! WP_Filesystem( $creds ) ) {
			return 'no_writing_rights';
		}
		global $wp_filesystem;
		if( empty( $wp_filesystem ) ){
			require_once ( ABSPATH . '/wp-admin/includes/file.php' );
			WP_Filesystem();
		}
		$upload_dir = wp_upload_dir();
		$asset_type = sanitize_key( $asset_type );
		if( !$wp_filesystem->is_dir( $upload_dir['basedir'].'/FDP' ) ){
			$wp_filesystem->mkdir( $upload_dir['basedir'].'/FDP' );
			if( !file_exists( $upload_dir['basedir'].'/FDP/index.php' ) ){
				$wp_filesystem->put_contents( $upload_dir['basedir'].'/FDP/index.php','' );
			}
		}
		if( !$wp_filesystem->is_dir( $upload_dir['basedir'].'/FDP/'.$asset_type ) ){
			$wp_filesystem->mkdir( $upload_dir['basedir'].'/FDP/'.$asset_type );
		}
		if( !file_exists( $upload_dir['basedir'].'/FDP/'.$asset_type.'/index.php' ) ){
			$wp_filesystem->put_contents( $upload_dir['basedir'].'/FDP/'.$asset_type.'/index.php','' );
		}
		$url_part = '/FDP/'.$asset_type.'/fdp-'.$post_id.'-'.substr( md5( $post_id.$asset_id ),0,8 ).'.'.$asset_type;
		if( $wp_filesystem->put_contents( $upload_dir['basedir'].$url_part,$content ) ){
			return $upload_dir['baseurl'].'/FDP/'.$asset_type.'/fdp-'.$post_id.'-'.substr( md5( $post_id.$asset_id ),0,8 ).'.'.$asset_type;
		}
		return false;
	}
}

function eos_dp_pro_get_user_headers( $args,$admin = true ){
	$cookies = array();
	if( $admin ){
		foreach ( $_COOKIE as $name => $value ) {
			$cookies[sanitize_key( $name )] = sanitize_text_field( $value );
		}
	}
	$headers = false;
	if( isset( $_POST['headers'] ) && !empty( $_POST['headers'] ) ){
		$headers = json_decode( sanitize_text_field( stripslashes( $_POST['headers'] ) ),true );
	}
	if( $headers ){
		$args['headers'] = $headers;
	}
	else{
		if( isset( $_SERVER['HTTP_AUTHORIZATION'] ) && !empty( $_SERVER['HTTP_AUTHORIZATION'] ) ){
			$args['headers'] = array(
					'Authorization' => sanitize_text_field( $_SERVER['HTTP_AUTHORIZATION'] )
			);
		}
		elseif( isset( $_SERVER['PHP_AUTH_USER'] ) && isset( $_SERVER['PHP_AUTH_PW'] ) && !empty( $_SERVER['PHP_AUTH_USER'] ) ){
			$credentials = base64_encode( $_SERVER['PHP_AUTH_USER'].':'.$_SERVER['PHP_AUTH_PW'] );
			$args['headers'] = array(
				'Authorization' => sanitize_text_field( 'Basic '.$credentials )
			);
		}
	}
	$args['headers']['Accept-Encoding'] = 'gzip, deflate';
	$args['cookies'] = $cookies;
	return $args;
}
