<?php
defined( 'ABSPATH' ) || exit; // Exit if accessed directly.

/**
 * Register hits per minute.
 *
 * @since 1.1.0.2
 *
 */
function fdp_pro_register_last_hits() {
	$hits = get_site_transient( 'fdp_last_hits' );
	$time = time();
	if( $hits && is_array( $hits ) ){
		foreach( $hits as $hit ){
			if( $time > $hit + 60 ){
				unset( $hits[ array_search( $hit, $hits ) ] );
			}
		}
	}
	else{
		$hits = array();
	}
	$hits[] = absint( $time );
	set_site_transient( 'fdp_last_hits', $hits, 60 );
}

/**
 * Get number of hits per minute
 *
 * @since 1.1.0.2
 *
 */
function fdp_pro_get_hits_per_minute() {
	$hits = get_site_transient( 'fdp_last_hits' );
	if( $hits && is_array( $hits ) ){
		$hits = array_values( $hits );
		return absint( 600 * count( $hits ) / ( 1 + ( $hits[ count( $hits ) - 1 ] - $hits[0] ) ) ) / 10;
	}
	return false;
}

add_action( 'muplugins_loaded',function() {
	if( function_exists( 'eos_dp_filter_active_plugins' ) ) {
		eos_dp_filter_active_plugins( 'fdp_pro_disable_plugin_by_plugin',99999 );
	}
} );

add_filter( 'fdp_deactivation_callbacks', function( $callbacks ) {
	/**
	 * Add deactivation callbacks
	 *
	 * @since 1.1.0.3
	 *
	 */	
	$callbacks[ 'fdp_pro_disable_plugin_by_plugin' ] = 99999;
	return $callbacks;
} );

/**
 * Disable plugin if a specific plugin is disabled
 *
 * @since 1.1.0.2
 *
 */
function fdp_pro_disable_plugin_by_plugin( $plugins ) {
	global $eos_dp_paths, $eos_dp_debug;
	if(  $eos_dp_paths && !empty( $eos_dp_paths ) ) {
		$disabled_plugins = array_filter( $eos_dp_paths );
		$opts_keys = array( 'eos_dp_by_plugin', 'eos_dp_conflicts' );
		$info = isset( $eos_dp_debug['info'] ) && ! empty( $eos_dp_debug['info'] ) ? $eos_dp_debug['info'] : array();
		foreach( $opts_keys as $opts_key ) {
			$opts = eos_dp_get_option( $opts_key );
			$plugins_array = 'eos_dp_by_plugin' === $opts_key ? $disabled_plugins : $plugins;
			if( $opts && is_array( $opts ) && ! empty( $opts ) ) {
				foreach( $opts as $plugin => $disabled_by_plugin_string ) {
					if( in_array( $plugin, $plugins_array ) && '' !== $disabled_by_plugin_string ) {
						$disabled_by_plugin = array_filter( explode( ',', $disabled_by_plugin_string ) );
						foreach( $disabled_by_plugin as $p ) {
							if( $p !== $plugin && !in_array( $p, $eos_dp_paths ) ) {
								$eos_dp_paths[] = $p;
								if( in_array( $p, $plugins ) ) {
									unset( $plugins[ array_search( $p, $plugins ) ] );
									if( 'eos_dp_by_plugin' === $opts_key ) {
										$info[] = sprintf( '%s disabled because %s is also disabled.',eos_dp_get_plugin_name_by_slug( $p ), eos_dp_get_plugin_name_by_slug(  $plugin ) );
									}
									elseif( 'eos_dp_conflicts' === $opts_key ) {
										$info[] = sprintf( '%s disabled because %s is active.',eos_dp_get_plugin_name_by_slug( $p ), eos_dp_get_plugin_name_by_slug(  $plugin ) );
									}
								}
							}
						}
					}
				}
			}
		}
		if( $eos_dp_debug && is_array( $eos_dp_debug ) && isset( $eos_dp_debug['info'] ) && isset( $info ) ) {
			$eos_dp_debug['info'] = array_unique( $info );
		}
	}
	return $plugins;
}


